function [npos,dpos]=agvsim(agv,sim,pos,va,vd)
%AGVSIM: Calculate the next position for an AGV
%
%                            [npos]=agvsim(agv,sim,pos,va,vd)
%
%          Where npos is the next position of the AGV
%          dpos is the change of the position of the AGV
%          agv is the description of the vehicle,
%          sim is the parameters of the simulation,
%          pos is the description of the agv's positon,
%          va,vd is the average and the differential speed
%                of the AGV's left and right drive
%
%          agv:
%
%            w  : distance of the sensor from the main point of the AGV
%            d  : distance of the wheels
%            ds : widt of the path sensing [-ds/2,ds/2]
%
%          sim:
%
%            dt   : time slice of the simulation
%            m    : full mass of the AGV
%            fmax : absolute limit for the accelerating force of the wheel
%            vmax  : absolute limit for the speed of the AGV
%            ekmax : absolute limit for the 
%                    distance between the main point and the path (ek)
%
%          pos, npos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          dpos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 5,error('Invalid number of arguments!');end

if length(agv) ~= 3
  error('Invalid AGV descriptor!');
end

if length(sim) ~= 5
  error('Invalid Simulation parameters!');
end

if length(pos) ~= 5
  error('Invalid Position!');
end

w = agv(1);  % the distance of the guide and the main point of the agv
d = agv(2);  % the distance of the wheels

dt = sim(1);  % the time slice
m = sim(2);  % full mass of the AGV
fmax = sim(3);  % absolute limit for the accelerating force of the wheel

xk = pos(1);  % the main point of the AGV
yk = pos(2);  %
fi = pos(3);  % angle of the AGV
vao = pos(4);  % previous speed
dfio = pos(5);  % previous angular velocity

% Calculating the forces of the wheels from the speeds required

fd = vd.*m./dt-dfio.*m./(d.*dt);  % fd = fl-fr
fa = va.*m./dt-vao.*m./dt;  % fa = (fl+fr)/2

fl = fa+fd./2;  % Accelerating force for the left wheel
fr = fa-fd./2;  % Accelerating force for the right wheel

% The forces are limited to fmax

if fl > fmax
  fl = fmax;
end;  % if

if fl < -fmax
  fl = -fmax;
end;  % if

if fr > fmax
  fr = fmax;
end;  % if

if fr < -fmax
  fr = -fmax;
end;  % if

fd = fl-fr;  % the difference of the forces
fa = (fl+fr)./2;  % the average of the forces

% Calculating the speeds of the wheels from the forces obtained

vd = dt.*fd./m+dfio./d;  % differential speed
va = dt.*fa./m+vao;      % average speed

% Calculating the new position

if abs(vd) > eps

  ro = d.*va./vd;

  dfi = dt.*vd./d;

  pdx = ro.*(1-cos(dfi));
  pdy = ro.*sin(dfi);

else

  dfi = 0;

  pdx = 0;
  pdy = dt.*va;

end % if

dx = pdx.*cos(fi)+pdy.*sin(fi);
dy = -pdx.*sin(fi)+pdy.*cos(fi);

xkn = xk+dx;
ykn = yk+dy;
fin = fi+dfi;

if fin>pi
  fin=fin-2.*pi;
end  % if

if fin<=-pi
  fin=fin+2.*pi;
end  % if

npos(1) = xkn;  % the new main point of the AGV
npos(2) = ykn;  %
npos(3) = fin;  % the new angle of the AGV
npos(4) = va;  % previous speed (vao)
npos(5) = vd./d;  % previous angular velocity (dfio)

dpos(1) = pdx;  % the change of the main point of the AGV
dpos(2) = pdy;  %
dpos(3) = dfi;  % the change of the angle of the AGV

