function [npos,ev,ek]=agvsys(agv,sim,path,pos,va,vd)
%AGVSYS: Calculate the next position for an AGV
%        and the input parameters for the control
%
%                            [npos,ev,ek]=agvsys(agv,sim,path,pos,va,vd)
%
%          Where npos is the next position of the AGV
%          ev is the distance between the guide point and the path
%          ek is the distance between the main point and the path
%          agv is the description of the vehicle,
%          sim is the parameters of the simulation,
%          path is the description of the path,
%          pos is the description of the agv's positon,
%          va,vd is the average and the differential speed
%                of the AGV's left and right drive
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%
%          sim:
%
%            dt   : time slice of the simulation
%            m    : full mass of the AGV
%            fmax : absolute limit for the accelerating force of the wheel
%            vmax  : absolute limit for the speed of the AGV
%            ekmax : absolute limit for the
%                    distance between the main point and the path (ek)
%
%          path:
%
%            lx : length of the first streight
%            r  : radius
%            ly : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          pos, npos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          dpos: change of the position
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 6,error('Invalid number of arguments!');end

if length(agv) ~= 3
  error('Invalid AGV descriptor!');
end

if length(sim) ~= 5
  error('Invalid Simulation parameters!');
end

if length(path) ~= 3
  error('Invalid Path!');
end

if length(pos) ~= 5
  error('Invalid Position!');
end

[npos,dpos]=agvsim(agv,sim,pos,va,vd);

w = agv(1);  % the distance of the guide and the main point of the agv

                            % distance between the guide point and the path
ev = pathdist(agv,path,npos);
                            % distance between the main point and the path
evo = pathdist(agv,path,pos);

if abs(ev)<inf & abs(evo)<inf  % real distance

  pdx = dpos(1);  % the change of the main point of the AGV
  pdy = dpos(2);  %
  dfi = dpos(3);  % the change of the angle of the AGV

  alf = atan2(pdy,pdx);  % atan(pdy./pdx)

  dk = sqrt(pdx.^2+pdy.^2);  % distance of the old and new main point

  dkx = -dk.*cos(alf+dfi);  % components of the old main point
  dky = -dk.*sin(alf+dfi);

  evox = -dkx+w.*sin(dfi)+evo.*cos(dfi);  % previous main point
  evoy =  dky+w.*cos(dfi)-evo.*sin(dfi);

  tpsi = (ev-evox)./(w-evoy);  % tan(psi)

  ekm = ev-w.*tpsi;  % distance in direction of wheel axle

  psi = atan2((ev-evox),(w-evoy));  % atan(tpsi)

  ek = ekm.*cos(psi);  % distance between the main point and the path

else  % not a real distance

  ek = inf;

end  % if

