function [nrunsp] = clrunsp(unsp,runsp,rl,rr,rm,agv,pos)
%CLRUNSP: Add new unsecure points and clean up the unsecure points matrix
%
%          [nrunsp] = clrunsp(unsp,runsp,rl,rr,rm,agv,pos)
%
%          Where unsp are the new unsecure points to add,
%          runsp is the unsecure points matrix,
%          rl,rr,rm are measured distances of the left, 
%            right and middle ultrasonic sensors,
%          agv is the description of the vehicle,
%          pos is the description of the agv positon.
%
%          rl,rr,rm:
%
%            r : measured distance
%            x,y : position of the measurement
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          unsp, runsp, nrunsp matrix of:
%
%            x,y : position of the approximated unsecure points
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 7,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 5
  error('Invalid AGV position!');
end

  % Parameters of the AGV

wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

hw = wagv./2;  % half width of the AGV
hl = lagv./2;  % half length of the AGV

x0 = pos(1);  % position of the main point of the AGV
y0 = pos(2);
fi = pos(3);  % angle of the AGV

  % Parameters of adding new unsecure points to the matrix

  % If there are unsecure point closer 
  % than minunspd in the matrix, do not add it

minunspd = wagv./30;  

minunspds = minunspd.^2;

  % Parameters of the cleaning up process

  % Unsecure point farther from the main point 
  % than maxunspd are uninteresting

maxunspd = lagv;  

maxunspds = maxunspd.^2;

  % Tolerance rate of the ultrasonic measurement

ustr = 1 + 1./100;

  % Positions of the ultrasonic sensors

sinfi = sin(fi);
cosfi = cos(fi);

  % Middle sensor

xmf = x0 + hl.*sinfi;
ymf = y0 + hl.*cosfi;

  % Rear middle point

xmr = x0 - hl.*sinfi;
ymr = y0 - hl.*cosfi;

  % Left sensor

xl = xmf - hw.*cosfi;
yl = ymf + hw.*sinfi;

  % Right sensor

xr = xmf + hw.*cosfi;
yr = ymf - hw.*sinfi;

  % Adding new elements and the clean up cycle

  % add new elements if they are not closer than minunspd
  % (check the latest (first) two element only)

[m,n] = size(runsp);

k = size(unsp,1);  % number of rows

if k>0 & m==0  % there were no previous elements

  [runsp] = [unsp(1,:);[runsp]];  % add the first point

end  % if

[m,n] = size(runsp);

if k>0 & m>0

  for j=1 : k

    dus = [inf,inf];  % clear temp variable

    [m,n] = size(runsp);

    for i=1 : min(m,3)  % checked number (2) + 1 (for the first new point)

      x = runsp(i,1);  % the actual unsecure point in the matrix
      y = runsp(i,2);

      xu = unsp(j,1);  % the unsecure point to add
      yu = unsp(j,2);

      dus(i) = (x-xu).^2+(y-yu).^2;

    end  % for i

    if min(dus)>=minunspds  % add to the record

      [runsp] = [unsp(j,:);[runsp]];  % add to the front

    end  % if

  end  % for j

end  % if

  % The clean up cycle (cleaning up runsp to nrunsp)

[m,n] = size(runsp);

nrunsp = [];  % clear the new record

if m > 0  % the record is not empty

  for i=1 : m  % check all the points

    rem = 0;  % temp variable (rem=1 to remove the point)

    x = runsp(i,1);  % the actual unsecure point
    y = runsp(i,2);

      % Remove if it is farther than maxunspd

    if ((x-x0).^2+(y-y0).^2) > maxunspds

      rem = 1;  % temp variable (rem=1 to remove the point)

    end  % if

      % Remove if it is under the actual position of the AGV

    a = (x-xmr).^2+(y-ymr).^2;
    b = (x-xmf).^2+(y-ymf).^2;

    da = (a-b+lagv.^2)./(2.*lagv);  % distance from the rear mid. point
    m = sqrt(a-da.^2);  % distance from the middle axes

      % The point is covered by the AGV:

    if m <= hw & 0<=da & da<=lagv

      rem = 1;  % temp variable (rem=1 to remove the point)

    end  % if

      % Remove if it is visible and closer than the sensed distances

      % Calculating the distance from the left sensor

    dx = x-xl;
    dy = y-yl;

    alf = atan2(dx,dy);  % atan((x-xl)./(y-yl)) the angle of the point
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

    if -pi<=ang & ang<=pi./2  % the point is visible

      rlp = sqrt(dx.^2+dy.^2);  % the distance

      if rlp < rl(1)./ustr  % remove the point

        rem = 1;  % temp variable (rem=1 to remove the point)

      end  % if

    end  % if

      % Calculating the distance from the right sensor

    dx = x-xr;
    dy = y-yr;

    alf = atan2(dx,dy);  % atan((x-xr)./(y-yr)) the angle of the point
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

    if -pi./2<=ang & ang<=pi  % the point is visible

      rrp = sqrt(dx.^2+dy.^2);  % the distance

      if rrp < rr(1)./ustr  % remove the point

        rem = 1;  % temp variable (rem=1 to remove the point)

      end  % if

    end  % if

      % Calculating the distances from the middle sensor

    dx = x-xmf;
    dy = y-ymf;

    alf = atan2(dx,dy);  % atan((x-xmf)./(y-ymf)) the angle of the point
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

    if -pi./2<=ang & ang<=pi./2  % the point is visible

      rmp = sqrt(dx.^2+dy.^2);  % the distance

      if rmp < rm(1)./ustr  % remove the point

        rem = 1;  % temp variable (rem=1 to remove the point)

      end  % if

    end  % if

    if rem ~=1  % if it still valid, add to the new record

      nrunsp = [nrunsp;[x,y]];  % cleared unsecure point record

    end  % if

  end  % for i

end  % if

