function [UD,VED,Rva,Rvd]=gagvflcc
% GAGVFLCC: Generate parmeters for Fuzzy Path Tracking and 
%                        Collision Avoidance
%              by interpolation in vague environment of an AGV
%
%                     [UD,VED,Rva,Rvd]=gagvflcc
%
%          Where [UD,VED,Rva,Rvd] are the FLC parameters.
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

% The universes considered for the control are:

U = [-1:.01:1];   % Components of the universe of discourse
UP = [0:.005:1];

Uev = U;          % Distance of the path from the guide point of the AGV
Uek = U;          % Distance of the path from the main point of the AGV

Url = UP;         % Distance measured by the left ultrasonic sensor
Urr = UP;         % Distance measured by the right ultrasonic sensor
Urm = UP;         % Distance measured by the middle ultrasonic sensor
Ualm = UP;        % Max left turn angle without side collision
Uarm = UP;        % Max riht turn angle without side collision

Uva = UP;         % Average speed of the AGV
Uvd = U;          % Differential speed of the AGV

  % Universe of discourse

UD = [Uev;Uek;Url;Urr;Urm;Ualm;Uarm;Uva;Uvd];

% The piecewise linear scaling functions are
%  (by their break points or value):

% Distance of the path from the guide point

evnl = -1;       % negativ large
evnm = -.475;      % negativ middle
evz  = 0;        % zero
evpm = .475;       % positiv middle
evpl = 1;        % positiv large

SPev=[  -1,  6;
      evnm,  1.714;
      evpm,  1.714;
         1,  6];

% Distance of the path from the main point

eknl = -1;       % negativ large
eknm = -.44;      % negativ middle
ekz  = 0;        % zero
ekpm = .44;       % positiv middle
ekpl = 1;        % positiv large

SPek=[  -1,  2;
         0, 10;
         1,  2];  % Distance of the path from the main point


  % Distance measured by the ultrasonic sensors

rs = .5;        % small
rsd = 0;         % left scalling factor
rsu = 20;        % right scalling factor

rl = .611;         % large
rld = 20;        % left scalling factor
rlu = 0;         % right scalling factor

  % Distance measured by the left ultrasonic sensor

rls = rs;        % small
rll = rl;        % large

SPrl = [rs, rsd, rsu;
        rl, rld, rlu];

  % Distance measured by the right ultrasonic sensor

rrs = rs;        % small
rrl = rl;        % large

SPrr = SPrl;


  % Distance measured by the middle ultrasonic sensor

rms = .5;         % small
rmsd = 0;         % left scalling factor
rmsu = 20;        % right scalling factor

rml = .6;         % large
rmld = 20;         % left scalling factor
rmlu = 0;         % right scalling factor

SPrm = [rms, rmsd, rmsu;
        rml, rmld, rmlu];


  % Max turn angle without side collision

ams = 0;        % small
amsd = 0;         % left scalling factor
amsu = 50;        % right scalling factor

aml = .1;         % large
amld = 50;        % left scalling factor
amlu = 0;         % right scalling factor

  % Max left turn angle without side collision

alms = ams;        % small
alml = aml;        % large

SPalm = [ams, amsd, amsu;
         aml, amld, amlu];

  % Max right turn angle without side collision

arms = ams;        % small
arml = aml;        % large

SParm = SPalm;


  % Average speed

vaz = 0;        % zero
vas = 0.1;      % small
val = 1;        % positiv large

SPva=[    0, .187;
          1, 3.125];

  % Differential speed

vdnl = -1;       % negativ large
vdns = -.3;      % negativ small
vdz = 0;         % zerus
vdps = .3;       % positiv small
vdpl = 1;        % positiv large

SPvd=[    2   ];

% Generating the scaling functions

SFev = gsclfunc(Uev,SPev);  % Distance of the path from the guide point
SFek = gsclfunc(Uek,SPek);  % Distance of the path from the main point

SFrl = gscnfunc(Url,SPrl); % Distance measured by the left ultrasonic sensor
SFrr = gscnfunc(Urr,SPrr); % Distance measured by the right ultrasonic sensor
SFrm = gscnfunc(Urm,SPrm); % Distance measured by the middle ultrasonic sensor
SFalm = gscnfunc(Ualm,SPalm); % Max left turn angle without side collision
SFarm = gscnfunc(Uarm,SParm); % Max riht turn angle without side collision
SFva = gsclfunc(Uva,SPva);  % Average speed
SFvd = gsclfunc(Uvd,SPvd);  % Differential speed

% Generating the vague environment

VEev = gvagenv(Uev,SFev);  % Distance of the path from the guide point
VEek = gvagenv(Uek,SFek);  % Distance of the path from the main point

VErl = gvagenv(Url,SFrl); % Distance measured by the left ultrasonic sensor
VErr = gvagenv(Urr,SFrr); % Distance measured by the right ultrasonic sensor
VErm = gvagenv(Urm,SFrm); % Distance measured by the middle ultrasonic sensor
VEalm = gvagenv(Ualm,SFalm); % Max left turn angle without side collision
VEarm = gvagenv(Uarm,SFarm); % Max riht turn angle without side collision

VEva = gvagenv(Uva,SFva);  % Average speed
VEvd = gvagenv(Uvd,SFvd);  % Differential speed

  % Vague universe of discourse

VED = [VEev;VEek;VErl;VErr;VErm;VEalm;VEarm;VEva;VEvd];

%
% The table of rules for control of differential and resultant speed;
%

% The Rulebases

  % The rules are rows Rvd = [ev,ek,rl,rr,rm,alm,arm,vd;...]

  % NaN: (Not a number) independent from the antecedent

x = nan;  % Independency from the rule antacedent

%Rvd= [ ev  ,  ek  , rl , rr , rm , alm , arm , vd ;...]

Rvd = [ evnm,  ekz ,  x ,  x ,  x ,   x , arml, vdpl;
        evpm,  ekz ,  x ,  x ,  x , alml,   x , vdnl;

        evnl,   x  ,  x ,  x ,  x ,   x , arml, vdpl;
        evpl,   x  ,  x ,  x ,  x , alml,   x , vdnl;
 
        evz, eknm ,  x , rrl, rml ,   x , arml, vdps;
        evz, eknm ,  x , rrs, rms ,   x ,   x , vdnl;

        evz, ekpm , rll,  x , rml , alml,   x , vdns;
        evz, ekpm , rls,  x , rms ,   x ,   x , vdpl;

        evz,  ekz , rll, rrs, rms ,  x ,  x , vdnl;
        evz,  ekz , rls, rrl, rms ,  x ,  x , vdpl;

        evnm, ekpm , rll,  x , rml , alml,   x , vdz;
        evpm, eknm ,  x , rrl, rml ,   x , arml, vdz ];



Rvx = [ evnm,  ekz ,  x ,  x ,  x  ,   x ,  x , vdpl;

        evpm,  ekz ,  x ,  x ,  x  ,  x ,   x , vdnl;

        evnl,  x  ,  x ,  x ,  x  ,  x ,  x , vdpl;
        evpl,  x  ,  x ,  x ,  x  ,  x ,  x , vdnl;

        evz, eknm ,  x ,  x ,  x  ,   x ,  x , vdps;

        evz, ekpm ,  x ,  x ,  x  ,  x ,   x , vdns;

        evnm, ekpm ,  x ,  x ,  x ,  x ,   x , vdns;

        evpm, eknm ,  x ,  x ,  x ,   x ,  x , vdps;

        evnm, ekpl ,  x ,  x ,  x  ,  x ,  x , vdpl;
        evpm, eknl ,  x ,  x ,  x  ,  x ,  x , vdnl ];


  % The rules are rows Rva = [ev,ek,rm,va;...]

  %  x=NaN : (Not a number) independent from the antecedent

%Rva= [ ev  ,  ek , rl , rr , rm , va ;...]

Rva = [ evz ,  ekz, rll, rrl, rml, val;
        evnl, ekpl,  x ,  x ,  x , vaz;
        evpl, eknl,  x ,  x ,  x , vaz;
        evnl,  ekz,  x ,  x ,  x , vaz;
        evpl,  ekz,  x ,  x ,  x , vaz ];

