function [UDC,VEDC,RCSva,RCSvd,RCRva,RCRvd,RCLva,RCLvd]=gagvflcf
% GAGVFLCF: Generate parmeters for Collision Avoidance, 
%              with Symmetric, Right and Left Tendency
%              by interpolation in vague environment of an AGV
%
%                     [UDC,VEDC,RCSva,RCSvd,RCRva,RCRvd,RCLva,RCLvd]=gagvflcf
%
%          Where [UDC,VEDC,RCSva,RCSvd,RCRva,RCRvd,RCLva,RCLvd] are the FLC parameters.
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

% The universes considered for the control are:

U = [-1:.01:1];   % Components of the universe of discourse
UP = [0:.005:1];

Url = UP;         % Distance measured by the left ultrasonic sensor
Urr = UP;         % Distance measured by the right ultrasonic sensor
Urm = UP;         % Distance measured by the middle ultrasonic sensor
Ualm = UP;        % Max left turn angle without side collision
Uarm = UP;        % Max riht turn angle without side collision

Uva = UP;         % Average speed of the AGV
Uvd = U;          % Differential speed of the AGV

  % Universe of discourse

UDC = [Url;Urr;Urm;Ualm;Uarm;Uva;Uvd];

% The piecewise linear scaling functions are
%  (by their break points or value):

  % Distance measured by the ultrasonic sensors

rz = 0;        % zerus
rzd = 0;         % left scalling factor
rzu = 10;        % right scalling factor

rs = .4;        % small
rsd = 10;         % left scalling factor
rsu = 20;        % right scalling factor

rl = .611;         % large
rld = 20;        % left scalling factor
rlu = 0;         % right scalling factor

  % Distance measured by the left ultrasonic sensor

rlz = rz;        % zerus
rls = rs;        % small
rll = rl;        % large

SPrl = [rz, rzd, rzu;
        rs, rsd, rsu;
        rl, rld, rlu];

  % Distance measured by the right ultrasonic sensor

rrz = rz;        % zerus
rrs = rs;        % small
rrl = rl;        % large

SPrr = SPrl;

  % Distance measured by the middle ultrasonic sensor

rms = .5;         % small
rmsd = 0;         % left scalling factor
rmsu = 20;        % right scalling factor

rml = .6;         % large
rmld = 20;         % left scalling factor
rmlu = 0;         % right scalling factor

SPrm = [rms, rmsd, rmsu;
        rml, rmld, rmlu];

  % Max turn angle without side collision

ams = 0;          % small
amsd = 0;         % left scalling factor
amsu = 50;        % right scalling factor

aml = .15;         % large
amld = 50;        % left scalling factor
amlu = 0;         % right scalling factor

  % Max left turn angle without side collision

alms = ams;        % small
alml = aml;        % large

SPalm = [ams, amsd, amsu;
         aml, amld, amlu];

  % Max right turn angle without side collision

arms = ams;        % small
arml = aml;        % large

SParm = SPalm;


  % Average speed

vaz = 0;        % zero
vas = 0.1;      % small
val = 1;        % positiv large

SPva=[    0, .187;
          1, 3.125];

  % Differential speed

vdnl = -1;       % negativ large
vdns = -.3;      % negativ small
vdnvs = -.1;     % negativ very small
vdz = 0;         % zerus
vdpvs = .1;      % positiv very small
vdps = .3;       % positiv small
vdpl = 1;        % positiv large

SPvd=[    2   ];

% Generating the scaling functions

SFrl = gscnfunc(Url,SPrl); % Distance measured by the left ultrasonic sensor
SFrr = gscnfunc(Urr,SPrr); % Distance measured by the right ultrasonic sensor
SFrm = gscnfunc(Urm,SPrm); % Distance measured by the middle ultrasonic sensor
SFalm = gscnfunc(Ualm,SPalm); % Max left turn angle without side collision
SFarm = gscnfunc(Uarm,SParm); % Max riht turn angle without side collision
SFva = gsclfunc(Uva,SPva);  % Average speed
SFvd = gsclfunc(Uvd,SPvd);  % Differential speed

% Generating the vague environment

VErl = gvagenv(Url,SFrl); % Distance measured by the left ultrasonic sensor
VErr = gvagenv(Urr,SFrr); % Distance measured by the right ultrasonic sensor
VErm = gvagenv(Urm,SFrm); % Distance measured by the middle ultrasonic sensor
VEalm = gvagenv(Ualm,SFalm); % Max left turn angle without side collision
VEarm = gvagenv(Uarm,SFarm); % Max riht turn angle without side collision
VEva = gvagenv(Uva,SFva);  % Average speed
VEvd = gvagenv(Uvd,SFvd);  % Differential speed

  % Vague universe of discourse

VEDC = [VErl;VErr;VErm;VEalm;VEarm;VEva;VEvd];

%
% The table of rules for control of differential and resultant speed;
%

% The Rulebases

  % The rules are rows Rvd = [ev,ek,rl,rr,rm,alm,arm,vd;...]

  % NaN: (Not a number) independent from the antecedent

x = nan;  % Independency from the rule antacedent

% Symmetric

%Rvd= [    rl , rr , rm , alm , arm , vd ;...]

RCSvd = [    x, rrz,   x , alml,   x , vdnl;
           rlz,   x,   x ,    x, arml, vdpl;
             x, rrz, rml , alms,    x, vdnvs;
           rlz,   x, rml ,    x, arms, vdpvs];

ORCSvd = [    x, rrz,   x , alml,   x , vdnl;
           rlz,   x,   x ,    x, arml, vdpl;
             x, rrz, rml , alms,    x, vdnvs;
           rlz,   x, rml ,    x, arms, vdpvs];

O2RCSvd = [   x , rrl, rml ,   x , arml, vdps;
           rll,  x , rml , alml,   x , vdns;
             x, rrz,   x , alml,   x , vdnl;
           rlz,   x,   x ,    x, arml, vdpl;
             x, rrz, rml , alms,    x, vdnvs;
           rlz,   x, rml ,    x, arms, vdpvs];

%Rva= [   rl , rr , rm , va ;...]

RCSva = [ rll, rrl, rml, val;
           x ,  x , rms, vas  ];

% Right tendency

[RCRvd] = [RCSvd;...
       [   x ,   x,   x ,   x , arms, vdz;...
           x , rrl, rml ,   x , arml, vdpl]];

RCRva = RCSva;

% Left tendency

[RCLvd] = [RCSvd;...
       [   x ,   x,   x , alms,   x , vdz;...
          rll,  x , rml , alml,   x , vdnl]];

RCLva = RCSva;



