function [UD,VED,Rva,Rvd]=gagvflcv
% GAGVFLCV: Generate parmeters for Fuzzy Path Tracking
%              by interpolation in vague environment of an AGV
%
%                     [UD,VED,Rva,Rvd]=gagvflcv
%
%          Where [UD,VED,Rva,Rvd] are the FLC parameters.
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

% The universes considered for the control are:

U = [-1:.01:1];   % Components of the universe of discourse
Uev = U;          % Distance of the path from the guide point of the AGV
Uek = U;          % Distance of the path from the main point of the AGV
Uva = [0:.005:1]; % Average speed of the AGV
Uvd = U;          % Differential speed of the AGV

UD = [Uev;Uek;Uva;Uvd];  % Universe of discourse

% The piecewise linear scaling functions are (by their break points or value):

SPev=[  -1,  5;
       -.5,  2;
        .5,  2;
         1,  5 ]; % Distance of the path from the guide point

SPek=[  -1,  2;
         0, 10;
         1,  2]; % Distance of the path from the main point

SPva=[    0, 1;
          1, 2 ]; % Average speed of the AGV

SPvd=[    2    ];  % Differential speed of the AGV

% Generating the scaling functions

SFev = gscfunc(Uev,SPev);  % Distance of the path from the guide point
SFek = gscfunc(Uek,SPek);  % Distance of the path from the main point
SFva = gscfunc(Uva,SPva);  % Average speed
SFvd = gscfunc(Uvd,SPvd);  % Differential speed

% Generating the vague environment

VEev = gvagenv(Uev,SFev);  % Distance of the path from the guide point
VEek = gvagenv(Uek,SFek);  % Distance of the path from the main point
VEva = gvagenv(Uva,SFva);  % Average speed
VEvd = gvagenv(Uvd,SFvd);  % Differential speed

VED = [VEev;VEek;VEva;VEvd];  % Vague universe of discourse

% The fuzzy values described by points in vague environment are:

% Distance of the path from the guide point

evnl = -1;       % negativ large
evnm = -.5;      % negativ middle
evz  = 0;        % zero
evpm = .5;       % positiv middle
evpl = 1;        % positiv large

% Distance of the path from the main point

eknl = -1;       % negativ large
eknm = -.5;      % negativ middle
ekz  = 0;        % zero
ekpm = .5;       % positiv middle
ekpl = 1;        % positiv large

% Average speed

vaz  = 0;        % zero
val = 1;         % positiv large

% Differential speed

vdnl = -1;       % negativ large
vdns = -.2;      % negativ small
vdps = .2;       % positiv small
vdpl = 1;        % positiv large

%
% The table of rules for control of differential and resultant speed;
%
% Differential speed
%
%  Rvd :   ev =
%          :  NL  NM   Z  PM  PL
%       ------------------------
%  ek = NL :              nl
%       NM :  pl      ps  ps  nl
%       Z  :      pl      nl
%       PM :  pl  ns  ns      nl
%       PL :      pl
%
%
% Average speed
%
%  Rva :   ev =
%          :  NL  NM   Z  PM  PL
%       ------------------------
%  ek = NL :                   z
%       NM :
%        Z :   z       l       z
%       PM :
%       PL :   z
%

% The Rulebases

  % The rules are rows Rvd = [ev,ek,vd;...]

Rvd = [ evnm,  ekz, vdpl;
        evpm,  ekz, vdnl;

        evnl, eknm, vdpl;
        evnl, ekpm, vdpl;

        evpl, eknm, vdnl;
        evpl, ekpm, vdnl;

         evz, eknm, vdps;
         evz, ekpm, vdns;

        evnm, ekpm, vdns;
        evpm, eknm, vdps;

        evnm, ekpl, vdpl;
        evpm, eknl, vdnl ];

  % The rules are rows Rva = [ev,ek,va;...]

Rva = [  evz,  ekz, val;
        evnl, ekpl, vaz;
        evpl, eknl, vaz;
        evnl,  ekz, vaz;
        evpl,  ekz, vaz  ];

