function [unsp]=geunsecp(uobs1,uobs2,uobsm,agv,pos)
%GEUNSECP: Generate unsecure points from two ultrasonic observations
%
%          [unsp]=geunsecp(uobs1,uobs2,agv,pos)
%
%          Where uobs1, uobs2 are the two ultrasonic observations,
%          uobsm is the middle ultrasonic observation,
%          unsp is the approximated unsecure point matrix,
%          agv is the description of the vehicle,
%          pos is the description of the agv positon.
%
%          uobs1, uobs2, uobsm:
%
%            r : the measured distance to an obstacle
%            x,y : position of the ultrasonic observation
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          unsp:
%
%            x,y : position of the approximated unsecure points
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 5,error('Invalid number of arguments!');end

if length(uobs1) ~= 3
  error('Invalid ultrasonic observation!');
end

if length(uobs2) ~= 3
  error('Invalid ultrasonic observation!');
end

if length(uobsm) ~= 3
  error('Invalid ultrasonic observation!');
end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 5
  error('Invalid AGV position!');
end

  % Parameters of the AGV

wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

hw = wagv./2;  % half width of the AGV
hl = lagv./2;  % half length of the AGV

x0 = pos(1);  % position of the main point of the AGV
y0 = pos(2);
fi = pos(3);  % angle of the AGV

  % Parameters of the unsecure points

  % maximal allowed distance from the front line to backwards

dusffmax1 = lagv./20;  % One usp in front, back distance limit for the other
dusffmax2 = lagv./2;  % Both usp in back, distance limit for them

dusffmaxs1 = dusffmax1.^2;
dusffmaxs2 = dusffmax2.^2;

  % Positions of the corner points of the AGV

sinfi = sin(fi);
cosfi = cos(fi);

  % Front middle

xmf = x0 + hl.*sinfi;
ymf = y0 + hl.*cosfi;

  % Rear middle point

xmr = x0 - hl.*sinfi;
ymr = y0 - hl.*cosfi;

  % Front left corner

xlf = xmf - hw.*cosfi;
ylf = ymf + hw.*sinfi;

  % Front right corner

xrf = xmf + hw.*cosfi;
yrf = ymf - hw.*sinfi;

  % local variables

r1 = uobs1(1);  % local variables
x1 = uobs1(2);
y1 = uobs1(3);

r2 = uobs2(1);  % local variables
x2 = uobs2(2);
y2 = uobs2(3);

rm = uobsm(1);  % local variables
xm = uobsm(2);
ym = uobsm(3);

  % Calculation the cut of the two circular observation
  %   as an approximated unsecure point
  % (only the "right side" cut is calculated)

dx = x2-x1;
dy = y2-y1;

d =  sqrt(dx.^2+dy.^2);  % the distance of the two observation
alf = atan2(dx,dy);  % atan(dx/dy)

sina = sin(alf);
cosa = cos(alf);

if d == 0  % special case, there are no movement

  m = nan;  % no approximated position
  d1 = nan;

elseif isinf(r1) | isinf(r2)  % special case, infinite distances

  m = nan;  % no approximated position
  d1 = nan;

elseif (r1+r2) < d  % special case, there are no cut

  m = min(r1,r2);  % approximated position
  d1 = d./2;

elseif (r1+d) < r2  % special case, there are no cut

  m = r1;  % approximated position
  d1 = 0;

elseif (r2+d) < r1  % special case, there are no cut

  m = r2;  % approximated position
  d1 = d;

else  % there are cut exists, the unsecure point could be approximated

  d1 = (r1.^2 - r2.^2 + d.^2)./(2.*d);
  m = sqrt(r1.^2 - d1.^2);

end  % if

if isnan(d1) | isnan(m)  % no approximated unsecure point

  unsp = [];

else  % the approximated unsecure points

  xdsina = x1 + d1.*sina;  % temp variables
  ydcosa = y1 + d1.*cosa;
  mcosa = m.*cosa;
  msina = m.*sina;

  ux1 = xdsina + mcosa;  % the first approximated unsecure point
  uy1 = ydcosa - msina;

  usp1 = 1;  % valid flag (init, valid)

  ux2 = xdsina - mcosa;  % the second approximated unsecure point
  uy2 = ydcosa + msina;

  usp2 = 1;  % valid flag (init, valid)

    % Remove if it is under the actual position of the AGV

  a = (ux1-xmr).^2+(uy1-ymr).^2;  % First point
  b = (ux1-xmf).^2+(uy1-ymf).^2;
  da = (a-b+lagv.^2)./(2.*lagv);  % distance from the rear mid. point
  m = sqrt(a-da.^2);  % distance from the middle axes

  if m <= hw & 0<=da & da<=lagv  % The point is covered by the AGV
    usp1 = 0;  % not valid
  end  % if

  a = (ux2-xmr).^2+(uy2-ymr).^2;  % Second point
  b = (ux2-xmf).^2+(uy2-ymf).^2;
  da = (a-b+lagv.^2)./(2.*lagv);  % distance from the rear mid. point
  m = sqrt(a-da.^2);  % distance from the middle axes

  if m <= hw & 0<=da & da<=lagv  % The point is covered by the AGV
    usp2 = 0;  % not valid
  end  % if

    % Selecting some of them

    % remove which is more back than the maximal allowed distance
    % (dusffmax) from the front line to backwards

    % Right front

    % the first approximated unsecure point

  dx = ux1-xrf;
  dy = uy1-yrf;

  a = dx.^2+dy.^2;  % distance^2 from the corner
  b = (ux1-xlf).^2+(uy1-ylf).^2;

  da = (a-b+wagv.^2)./(2.*wagv);  % projection to the side 
  m1 = a-da.^2;  % distance^2 from the front line

  alf = atan2(dx,dy);  % atan(dx./dy) the angle
  ang1 = alf-fi;

    % the second approximated unsecure point

  dx = ux2-xrf;
  dy = uy2-yrf;

  a = dx.^2+dy.^2;  % distance^2 from the corner
  b = (ux2-xlf).^2+(uy2-ylf).^2;

  da = (a-b+wagv.^2)./(2.*wagv);  % projection to the side 
  m2 = a-da.^2;  % distance^2 from the front line

  alf = atan2(dx,dy);  % atan(dx./dy) the angle
  ang2 = alf-fi;

    % Checking the validities

  if (-pi./2<=ang1 & ang1<=pi./2)
    uspf1 = 1;  % in front flag
  else
    uspf1 = 0;  % not in front flag
  end  % if

  if (-pi./2<=ang2 & ang2<=pi./2)
    uspf2 = 1;  % in front flag
  else
    uspf2 = 0;  % not in front flag
  end  % if

  if uspf1==1 & ((ux1-xm).^2+(uy1-ym).^2)<(rm.^2)  % the middle sensor can't see it
      usp1 = 0;  % not valid
  end  % if

  if uspf2==1 & ((ux2-xm).^2+(uy2-ym).^2)<(rm.^2)  % the middle sensor can't see it
      usp2 = 0;  % not valid
  end  % if

  if usp1+usp2==2 & uspf2==1 & uspf1==0 & m1>dusffmaxs1
    usp1 = 0;  % the unsp1 is not allowed (too far back)
  end  % if
  
  if usp1+usp2==2 & uspf1==1 & uspf2==0 & m2>dusffmaxs1
    usp2 = 0;  % the unsp2 is not allowed (too far back)
  end  % if

  if usp1+usp2==2 & uspf1+uspf2==0
    if m1>dusffmaxs2
      usp1 = 0;  % the unsp1 is not allowed (too far back)
    end  % if
    if m2>dusffmaxs2
      usp2 = 0;  % the unsp2 is not allowed (too far back)
    end  % if
  end  % if

  unsp = [];

  if usp1==1  % valid
    [unsp] = [unsp;[ux1,uy1]];  % the approximated unsecure point
  end  % if

  if usp2==1  % valid
    [unsp] = [unsp;[ux2,uy2]];  % the approximated unsecure point
  end  % if

end  % if


