function [agv,sim,path,statpar,pos]=iniagvsp(magv,rpath)
%INIAGVSP: Initialize the AGV simulation parameters
%
%            [agv,sim,path,statpar,pos]=iniagvsp(magv,rpath)
%
%          Where 
%          magv is the mass of the AGV,
%          rpath is the radius of the path,
%          agv is the description of the vehicle,
%          sim is the collection of the simulation parameters,
%          path is the description of the path,
%          statpar is the description of the docking station,
%          pos is the description of the initial agv positon.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          sim:
%
%            dt [s]: time slice of the simulation
%            m [kg]: full mass of the AGV
%            fmax [N]: absolute limit for the 
%                      accelerating force of the wheel
%            vmax [m/s]: absolute limit for the speed of the AGV
%            ekmax [m]: absolute limit for the distance between
%                       the main point and the path
%
%          Path:
%
%            lx : length of the first streight
%            r  : radius
%            ly : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          statpar:
%
%            demax : max error as a distance of the path 
%                    and the main point of the AGV 
%                    in the docking station
%            aemax : max error of the angle of the AGV 
%                    in the docking station
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 2,error('Invalid number of arguments!');end

if length(magv) ~= 1
  error('Invalid AGV mass value!');
end

if length(rpath) ~= 1
  error('Invalid path radius value!');
end

  % Generate the AGV parameters

[agv]=genagv;

  % Simulation parameters

  % dt [s]: time slice of the simulation

sim(1) = .03;

  % m [kg]: full mass of the AGV

sim(2) = magv;  % 570kg unloaded, 1570kg full loaded

  % fmax [N]: absolute limit for the accelerating force of the wheel

sim(3) = 3000;

  % vmax [m/s]: absolute limit for the speed of the AGV

sim(4) = 1.5;

  % ekmax [m]: absolute limit for the distance between
    % the main point and the path
    % the distance of the guide and the main point of the agv

sim(5) = agv(1);  

  % Parameters of the station

  % max error of the path main point distance in the station

demax = .01;

  % max error of the angle of the AGV in the station

aemax = .026;

statpar=[demax,aemax];  % Parameters of the docking station

  % Generate the path and initial position parameters

[path,pos]=genapath(agv,rpath);  % Circular arc and straight

