function [agv,sim,path,statpar,pos]=modpathr(sim,agvobj,pathobj)
%MODPATHR: Modify the path radius
%
%        [agv,sim,path,statpar,pos]=modpathr(sim,agvobj,pathobj)
%
%          Where agv is the description of the vehicle,
%          sim is the collection of the simulation parameters,
%          path is the description of the path,
%          statpar is the description of the docking station,
%          pos is the description of the initial agv positon,
%          agvobj is the AGV object,
%          pathobj is the path object.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          sim:
%
%            dt [s]: time slice of the simulation
%            magv [kg]: full mass of the AGV
%            fmax [N]: absolute limit for the 
%                      accelerating force of the wheel
%            vmax [m/s]: absolute limit for the speed of the AGV
%            ekmax [m]: absolute limit for the distance between
%                       the main point and the path
%
%          path:
%
%            lx    : length of the first streight
%            rpath : radius
%            ly    : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          statpar:
%
%            demax : max error as a distance of the path 
%                    and the main point of the AGV 
%                    in the docking station
%            aemax : max error of the angle of the AGV 
%                    in the docking station
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          agvobj:
%
%            agvobj=[body,...  % Body of the AGV
%              guidezone,...  % The guide zone
%              wheels,...  % The wheels
%              mainpos,...  % The main point
%              guidep,...  % The guide point
%              agvline,...  % The AGV line 
%                           % (line betweem the guige and main point)
%              sens];  % The ultrasonic sensors
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(sim) ~= 5
  error('Invalid simulation parameters!');
end

if length(agvobj) ~= 7
  error('Invalid AGV object!');
end

if length(pathobj) ~= 1
  error('Invalid path object!');
end

  % Modify the user interface

figNumber = gcf;  % The figure

hndlList = get(figNumber,'UserData');

radiusHndl = hndlList(5);

  % radius of the path

rpath = round(get(radiusHndl,'Value').*20)./20;  % radius of the path

set(radiusHndl,'Value',rpath);  % rounding

  %  Initialize the AGV simulation parameters

[agv,sim,path,statpar,pos]=iniagvsp(sim(2),rpath);

  % Plot the path and the AGV on the screen in its initial position

plotpagv(agv,path,pos,agvobj,pathobj);

