function [dist]=pathdist(agv,path,pos)
%PATHDIST: Calculate the distance from the path
%
%                            [dist]=pathdist(agv,path,pos)
%
%          Where dist is the distance of the guide point from the path,
%            (in the direction of the sensors)
%          agv is the description of the vehicle,
%          path is the description of the path,
%          pos is the description of the agv's positon.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing [-ds/2,ds/2]
%
%          path:
%
%            lx : length of the first streight
%            r  : radius
%            ly : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(agv) ~= 3
  error('Invalid AGV descriptor!');
end

if length(path) ~= 3
  error('Invalid Path!');
end

if length(pos) ~= 5
  error('Invalid Position!');
end

w = agv(1);  % the distance of the guide and the main point of the agv
ds = agv(3);  % the widt of the path sensing

hds = ds./2;
hdss = hds.^2;

lx = path(1);  % length of the first streight
r = path(2);  % radius
ly = path(3);  % length of the last streight

xk = pos(1);  % the main point of the AGV
yk = pos(2);  %
fi = pos(3);  % angle of the AGV

sinfi = sin(fi);
cosfi = cos(fi);

  % the guide point of the AGV

xv = xk+w.*sinfi;
yv = yk+w.*cosfi;

  % the two end points of the guide point of the AGV

  % right 

xv1 = xv + hds.*cosfi;
yv1 = yv - hds.*sinfi;

  % left

xv2 = xv - hds.*cosfi;
yv2 = yv + hds.*sinfi;

  % the slope of the guide zone

dxv = xv2-xv1;

if dxv~=0
  mv = (yv2-yv1)./(xv2-xv1);
else
  mv = inf;
end  % if


  % Checking the first streight

x1 = -lx;
y1 = -r;

x2 = 0;
y2 = -r;

  % the cut of the two lines

  % the slope of the first streight: m = 0

if isinf(mv)

  x = xv;
  y = y1;  % -r

elseif mv==0

  if yv == y1  % -r

    if x1<=xv & xv<=x2
      x = xv;  % on the line
      y = y1;  % -r
    end  % if

    if xv>x2 & (xv1<=x2 | xv2<=x2)
      x = x2;  % on the line
      y = y1;  % -r
    end  % if

  else

    x = inf;  % not exists
    y = inf;

  end  % if

else

  x = (y1-yv1+mv.*xv1)./mv;
  y = y1;  % -r

end  % if

  % Check if the cut is on the first streight

dfss = (x-xv).^2+(y-yv).^2;  % absolute distance square

if dfss<=hdss & (x1<=x & x<=x2 & y==y1)

    % distance from the first stright on the guide zone

    % absolute distance to the first straight

  dfs = sqrt(dfss);

    % sign of the distance

  if (x-xv2).^2+(y-yv2).^2 <= (x-xv1).^2+(y-yv1).^2

    sdfs = +1;  % sign of the distance

  else

    sdfs = -1;  % sign of the distance

  end  % if

else

  dfs = inf;  % not exists
  sdfs = +1;

end  % if


  % Checking the last streight

x1 = r;
y1 = 0;

x2 = r;
y2 = ly;

  % the cut of the two lines

  % the slope of the last streight: m = inf

if mv==0

  y = yv;
  x = x1;  % r

elseif isinf(mv)

  if xv == x1  % r

    if y1<=yv & yv<=y2
      y = yv;  % on the line
      x = x1;  % r
    end  % if

    if yv<y1 & (yv1>=y1 | yv2>=y1)
      y = y1;  % on the line
      x = x1;  % r
    end  % if

  else

    y = inf;  % not exists
    x = inf;

  end  % if

else

  y = yv1+mv.*(x1-xv1);
  x = x1;  % r

end  % if

  % Check if the cut is on the last streight

dlss = (x-xv).^2+(y-yv).^2;  % absolute distance square

if dlss<=hdss & (y1<=y & x==x1)  % enable y>y2 as it were longer

    % distance from the last stright on the guide zone

  dls = sqrt(dlss);  % absolute distance to the last straight

    % sign of the distance

  if (x-xv2).^2+(y-yv2).^2 <= (x-xv1).^2+(y-yv1).^2

    sdls = +1;  % sign of the distance

  else

    sdls = -1;  % sign of the distance

  end  % if

else

  dls = inf;  % not exists
  sdls = +1;

end  % if


  % Checking the curve

  % the cut of a line and a circle

k = yv1 - mv.*xv1;

a2 = (1 + mv.^2).*2;  % 2*a
b = 2.*k.*mv;
c = k.^2 - r.^2;

det = b.^2 - 2.*a2.*c;

if det==0 | abs(mv)>1e6  % the cuts are exists or mv=inf

  x = xv;
  ys = r.^2 - x.^2;

  if ys>=0

    y = -sqrt(ys);

  else

    y = inf;  % not exists
    x = inf;

  end  % if

    % Check if the cut is on the curve

  dcs = (x-xv).^2+(y-yv).^2;  % absolute distance square

  if dcs<=hdss & (-r<=y & y<=0 & 0<=x & x<=r)

      % distance of the curve and the cut

    dc = sqrt(dcs);  % absolute distance

      % sign of the distance

    if (x-xv2).^2+(y-yv2).^2 <= (x-xv1).^2+(y-yv1).^2

      sdc = +1;  % sign of the distance

    else

      sdc = -1;  % sign of the distance

    end  % if

  else

    dc = inf;  % not exists
    sdc = +1;  % sign of the distance

  end  % if

elseif det>0  % the cuts are exists, mv~=inf

  sdet = sqrt(det);

  x1 = (-b + sdet)./a2;
  y1 = yv1+mv.*(x1-xv1);

  x2 = (-b - sdet)./a2;
  y2 = yv1+mv.*(x2-xv1);

    % Check if the first cut is on the curve

  dc1s = (x1-xv).^2+(y1-yv).^2;  % absolute distance square

  if dc1s<=hdss & (-r<=y1 & y1<=0 & 0<=x1 & x1<=r)

      % distance of the curve and the first cut

    dc1 = sqrt(dc1s);  % absolute distance

      % sign of the distance

    if (x1-xv2).^2+(y1-yv2).^2 <= (x1-xv1).^2+(y1-yv1).^2

      sdc1 = +1;  % sign of the distance

    else

      sdc1 = -1;  % sign of the distance

    end  % if

  else

    dc1 = inf;  % not exists
    sdc1 = +1;  % sign of the distance

  end  % if

    % Check if the second cut is on the curve

  dc2s = (x2-xv).^2+(y2-yv).^2;  % absolute distance square

  if dc2s<=hdss & (-r<=y2 & y2<=0 & 0<=x2 & x2<=r)

      % distance of the curve and the second cut

    dc2 = sqrt(dc2s);  % absolute distance

      % sign of the distance

    if (x2-xv2).^2+(y2-yv2).^2 <= (x2-xv1).^2+(y2-yv1).^2

      sdc2 = +1;  % sign of the distance

    else

      sdc2 = -1;  % sign of the distance

    end  % if

  else

    dc2 = inf;  % not exists
    sdc2 = +1;  % sign of the distance

  end  % if

    % choosing the minimal one

  if dc1 == dc2 & sdc1~=sdc2

    dc = 0;  % symmetrical
    sdc = +1;  % sign of the distance

  elseif dc1 <= dc2

    dc = dc1;  % absolute distance
    sdc = sdc1;  % sign

  else

    dc = dc2;  % absolute distance
    sdc = sdc2;  % sign

  end  % if

else  % no cuts

  dc = inf;  % not exists
  sdc = +1;  % sign of the distance

end  % if


  % Choosing the minimal one from dfs, dls, dc

m = [dfs,dls,dc;sdfs,sdls,sdc];

[dist,n] = min(m(1,:));  % absolute distance

sdist = m(2,n);  % sign

  % check if there are more identical with different sign

if dist==dfs & sdist~=sdfs
  dist = 0;  % symmetrical
end  % if

if dist==dls & sdist~=sdls
  dist = 0;  % symmetrical
end  % if

if dist==dc & sdist~=sdc
  dist = 0;  % symmetrical
end  % if

dist = dist.*sdist;  % the signed distance

