function plagvres(agv,sim,path,obst,rtime,rpos,re,fspos)
%PLAGVRES: Plot the results of the AGV simulation
%
%            plagvres(agv,sim,path,obst,rtime,rpos,re,fspos)
%
%          Where agv is the description of the vehicle,
%          sim is the collection of the simulation parameters,
%          path is the description of the path,
%          obst is the description of the obstacles,
%          rtime is the recorded running time,
%          rpos is the recorded positions,
%          re is the recorded control parameters,
%          fspos is the first position 
%                fits the requirements of the station.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          sim:
%
%            dt [s]: time slice of the simulation
%            m [kg]: full mass of the AGV
%            fmax [N]: absolute limit for the 
%                      accelerating force of the wheel
%            vmax [m/s]: absolute limit for the speed of the AGV
%            ekmax [m]: absolute limit for the distance between
%                       the main point and the path
%
%          path:
%
%            lx : length of the first streight
%            r  : radius
%            ly : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          obst:
%
%            x,y : position of the obstacle
%            r  : radius of the obstacle
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%
%          rtime:
%
%            [rtime] = [rtime;[time]];  % running time
%
%          rpos:
%
%            [rpos]=[rpos;[pos(1),pos(2),pos(3)]];  % positions
%
%          re:   control parameters
%
%            [re] = [re;[ev,ek,rl,rr,rm,alm,arm,vd,va]];
%
%          ev is the distance between the guide point and the path
%          ek is the distance between the main point and the path
%          rl,rr,rm are measured distances of the left, 
%            right and middle ultrasonic sensors,
%          alm,arm are the left and right
%            maximal turn angle withouth side collision,
%          va,vd is the average and the differential speed
%                of the AGV's left and right drive
%
%          fspos:
%
%            fspos = [pos(1),pos(2),pos(3),ev,ek,time];  % first station
%            fspos = [0,0,0,0,0,-1];  % if there are no accaptable station
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 8,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(sim) ~= 5
  error('Invalid simulation parameters!');
end

if length(path) ~= 3
  error('Invalid path parameters!');
end

if length(fspos) ~= 6
  error('Invalid first station parameters!');
end


  % First plot, draw the lines only

  % open a window

figtrajNum = figure(...
	'Name','Trajectories of the Automated Guided Vehicle',... 
	'NumberTitle','on',...
	'Color',[0 0 0],...
	'BackingStore','off',...
	'NextPlot','add',...
	'Pointer','arrow',...
	'Units','normal',...
	'Position',[0 0 1 1],...
	'Resize','on',...
	'Visible','off');

axes( ...
  'Units','normalized', ...
  'Color',[0 0 0],...
  'XColor',[1 1 1],...
  'YColor',[1 1 1],...
  'Visible','on');

plagvlin(agv,rpos);  % Draw the AGV lines

  % first position fits the requirements of the station

hold on;

if fspos(6)~=-1  % if exists
  plot(fspos(1),fspos(2),'g*');
end  % if

  % Draw the path

pathp=gpathp(path,.1);  % Generate the points of the path

hold on;
plot(pathp(:,1),pathp(:,2),'y-');  % Plot the path

  % Plot the obstacles

plobst(obst);

  % Add title

title(['R=',num2str(path(2)),'[m]',...
       '   M=',num2str(sim(2)),'[kg]',...
       '   dS=',num2str(fspos(2)),'[m]',...
       '   tS=',num2str(fspos(6)),'[s]']);  % title

figure(figtrajNum);  % Draw the figure

axis('equal'); % equal and tight axis
%axis('image'); % equal and tight axis


  % Second plot, draw the lines and the AGVs

  % open a window

figagvtrNum = figure(...
	'Name','Trajectories and run of the Automated Guided Vehicle',... 
	'NumberTitle','on',...
	'Color',[0 0 0],...
	'BackingStore','off',...
	'NextPlot','add',...
	'Pointer','arrow',...
	'Units','normal',...
	'Position',[0 0 1 1],...
	'Resize','on',...
	'Visible','off');

axes( ...
  'Units','normalized', ...
  'Color',[0 0 0],...
  'XColor',[1 1 1],...
  'YColor',[1 1 1],...
  'Visible','on');

plagvlin(agv,rpos);  % Draw the AGV lines

plagvs(agv,rpos,5);  % Draw the AGVs (5 pieces)

  % first position fits the requirements of the station

hold on;
if fspos(6)~=-1  % if exists
  plot(fspos(1),fspos(2),'g*');
end  % if

  % Draw the path

hold on;
plot(pathp(:,1),pathp(:,2),'y-');  % Plot the path

  % Plot the obstacles

plobst(obst);

  % Add title

title(['R=',num2str(path(2)),'[m]',...
       '   M=',num2str(sim(2)),'[kg]',...
       '   dS=',num2str(fspos(2)),'[m]',...
       '   tS=',num2str(fspos(6)),'[s]']);  % title

figure(figagvtrNum);  % Draw the figure

axis('equal'); % equal and tight axis
%axis('image'); % equal and tight axis


  % Third plot, draw the inputs and the outputs of the FLC

  % open a window

figflcNum = figure(...
	'Name','Observations and conclusions of the FLC',... 
	'NumberTitle','on',...
	'Color',[0 0 0],...
	'BackingStore','off',...
	'NextPlot','add',...
	'Pointer','arrow',...
	'Units','normal',...
	'Position',[0 0 1 1],...
	'Resize','on',...
	'Visible','off');

axes( ...
  'Units','normalized', ...
  'Position',[0.1 0.4 0.75 0.55], ...
  'Visible','on');

plot(rtime,re(:,1),'r-');  % ev

hold on;
plot(rtime,re(:,2),'g-');  % ek

hold on;
plot(rtime,re(:,8),'m-');  % vd

hold on;
plot(rtime,re(:,9),'c-');  % va

trl=[];
rl=[];
trr=[];
rr=[];
trm=[];
rm=[];
talm=[];
alm=[];
tarm=[];
arm=[];

n = size(rtime,1);

for i=1 : fix(n./30) : n

  if re(i,3)<1  % plot it
    [trl] = [trl;[rtime(i)]];
    [rl] = [rl;[-re(i,3)]];
  end  % if

  if re(i,4)<1  % plot it
    [trr] = [trr;[rtime(i)]];
    [rr] = [rr;[re(i,4)]];
  end  % if

  if re(i,5)<1  % plot it
    [trm] = [trm;[rtime(i)]];
    [rm] = [rm;[re(i,5)]];
  end  % if

  if re(i,6)<1  % plot it
    [talm] = [talm;[rtime(i)]];
    [alm] = [alm;[-re(i,6)]];
  end  % if

  if re(i,7)<1  % plot it
    [tarm] = [tarm;[rtime(i)]];
    [arm] = [arm;[re(i,7)]];
  end  % if

end  % for i

hold on;
plot(trl,rl,'m-*');  % rl

hold on;
plot(trr,rr,'m-*');  % rr

hold on;
plot(trm,rm,'y-d');  % rm

hold on;
plot(talm,alm,'b-o');  % alm

hold on;
plot(tarm,arm,'b-o');  % alm

  % set black background and white axes

set(gca,...
  'Color',[0 0 0],...
  'XColor',[1 1 1],...
  'YColor',[1 1 1]);

axes( ...
  'Units','normalized', ...
  'Position',[0.1 0.05 0.75 0.3], ...
  'Visible','on');

plot(rtime,re(:,10),'c-');  % sc

hold on;
plot(rtime,re(:,11),'y-');  % ss

hold on;
plot(rtime,re(:,12),'r-');  % sr

hold on;
plot(rtime,re(:,13),'g-');  % sl

set(gca,...
  'Color',[0 0 0],...
  'XColor',[1 1 1],...
  'YColor',[1 1 1]);

title('Observations and conclusions of the FLC');  % title

  % Draw comments as text buttons

  % Information for all buttons

btnColor=[0 0 0];
textColor='w';
yInitPos=0.9;
xPos=0.85;
btnLen=0.1;
btnWid=0.05;
fonts=12;  % Font Size

  % Spacing between the button and the next command's label

spacing=0.015;
    
  % Text comment button (Observations)

btnNumber=1;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Observations:';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor',textColor,...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The ev button (The distance between the guide point and the path)

btnNumber=2;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Eg: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','r',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The ek button (The distance between the main point and the path)

btnNumber=3;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Em: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','g',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);
    
  % The Rl,Rr button (Left and right ultrasonic distance)

btnNumber=4;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Rl, Rr: * ';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','m',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Rm button (Middle ultrasonic distance)

btnNumber=5;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Rm: * ';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','y',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Alm, Arm button (left and right maximal turn angle)

btnNumber=6;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Alm, Arm: o ';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','b',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % Text comment button (Conclusions)

btnNumber=7;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Conclusions:';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor',textColor,...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The vd button (The differential speed conclusion)

btnNumber=8;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Vd: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','m',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The va button (The average speed conclusion)

btnNumber=9;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='Va: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','c',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % Text comment button (States)

btnNumber=10;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='States:';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor',textColor,...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Sc button (Path Tracking and Collision Avoidance)

btnNumber=11;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='S_P&C: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','c',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Ss button (Collision Avoidance)

btnNumber=12;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='S_C: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','y',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Sr button (Collision Avoidance and Right Tendency)

btnNumber=13;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='S_C&R: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','r',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % The Sl button (Collision Avoidance and Left tendency)

btnNumber=14;
yPos=yInitPos-(btnNumber-1)*(btnWid+spacing);
labelStr='S_C&L: -';

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'ForegroundColor','g',...
  'BackgroundColor',btnColor,...
  'FontSize',fonts,...
  'String',labelStr);

  % Draw the figure

figure(figflcNum);  % Draw the figure

