function plotagv(agv,pos,agvobj)
%PLOTAGV: Plot an AGV on screen
%
%                    plotagv(agv,pos,agvobj)
%
%          Where agv is the description of the vehicle,
%          pos is the description of the actual agv's positon,
%          agvobj is the AGV object.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%
%          agvobj:
%
%            agvobj=[body,...  % Body of the AGV
%              guidezone,...  % The guide zone
%              wheels,...  % The wheels
%              mainpos,...  % The main point
%              guidep,...  % The guide point
%              agvline,...  % The AGV line 
%                           % (line betweem the guige and main point)
%              sens];  % The ultrasonic sensors
%
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 3
  error('Invalid AGV actual position!');
end

if length(agvobj) ~= 7
  error('Invalid AGV object!');
end

  % The AGV object

body = agvobj(1);  % Body of the AGV
guidezone = agvobj(2);  % The guide zone
wheels = agvobj(3);  % The wheels
mainpos = agvobj(4);  % The main point
guidep = agvobj(5);  % The guide point
agvline = agvobj(6);  % The AGV line
sens = agvobj(7);  % The ultrasonic sensors

  % The AGV parameters

w = agv(1);  % the distance of the guide and the main point of the agv
d = agv(2);  % distance of the wheels
ds = agv(3);  % widt of the path sensing
wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

db = lagv./2;  % half length of the AGV
wb = wagv./2;  % half width of the AGV
ww = d./30;  % width of the wheel of the AGV

  % Parts of the AGV

xk = pos(1);
yk = pos(2);
fi = pos(3);

sinfi = sin(fi);
cosfi = cos(fi);

  % Body of the AGV

wbsinfi = wb.*sinfi;
wbcosfi = wb.*cosfi;
dbsinfi = db.*sinfi;
dbcosfi = db.*cosfi;

xmf = xk+dbsinfi;
ymf = yk+dbcosfi;
xmr = xk-dbsinfi;
ymr = yk-dbcosfi;


pax = xmf-wbcosfi;  % body
pay = ymf+wbsinfi;
pbx = xmf+wbcosfi;
pby = ymf-wbsinfi;
pcx = xmr+wbcosfi;
pcy = ymr-wbsinfi;
pdx = xmr-wbcosfi;
pdy = ymr+wbsinfi;

bodyx = [pax,pbx,pcx,pdx,pax];
bodyy = [pay,pby,pcy,pdy,pay];

set(body,...
  'xdata',bodyx,'ydata',bodyy);

  % The guide zone

wsinfi = w.*sinfi;
wcosfi = w.*cosfi;
dssinfi = ds.*sinfi./2;
dscosfi = ds.*cosfi./2;

spax = xk+wsinfi-dscosfi;  % guide zone
spay = yk+wcosfi+dssinfi;
spbx = xk+wsinfi+dscosfi;
spby = yk+wcosfi-dssinfi;

gsx = [spax,spbx];
gsy = [spay,spby];

set(guidezone,...
  'xdata',gsx,'ydata',gsy);

  % The wheels

wwsinfi = ww.*sinfi;
wwcosfi = ww.*cosfi;
dsinfi = d.*sinfi./2;
dcosfi = d.*cosfi./2;

wpax = xk+wwsinfi-dcosfi;  % wheels
wpay = yk+wwcosfi+dsinfi;
wpbx = xk+wwsinfi+dcosfi;
wpby = yk+wwcosfi-dsinfi;
wpcx = xk-wwsinfi+dcosfi;
wpcy = yk-wwcosfi-dsinfi;
wpdx = xk-wwsinfi-dcosfi;
wpdy = yk-wwcosfi+dsinfi;

wpx = [wpax,wpbx,wpcx,wpdx];
wpy = [wpay,wpby,wpcy,wpdy];

set(wheels,...
  'xdata',wpx,'ydata',wpy);

  % The main point

set(mainpos,...
  'xdata',xk,'ydata',yk);

  % The guide point

xv = xk+wsinfi;  % position of the guide point
yv = yk+wcosfi;

set(guidep,...
  'xdata',xv,'ydata',yv);

  % The AGV line

set(agvline,...
  'xdata',[xk,xv],'ydata',[yk,yv]);

  % The ultrasonic sensors

sensx = [pax,pbx,xmf];  % ultrasonic sensors
sensy = [pay,pby,ymf];

set(sens,...
  'xdata',sensx,'ydata',sensy);

  % Drawing the AGV

drawnow;

