function plotpagv(agv,path,pos,agvobj,pathobj)
%PLOTPAGV: Plot the path and the AGV on the screen in its initial position
%
%          plotpagv(agv,path,pos,agvobj,pathobj)
%
%          Where agv is the description of the vehicle,
%          path is the description of the path,
%          pos is the description of the initial agv positon,
%          agvobj is the AGV object,
%          pathobj is the path object.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          path:
%
%            lx    : length of the first streight
%            rpath : radius
%            ly    : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          agvobj:
%
%            agvobj=[body,...  % Body of the AGV
%              guidezone,...  % The guide zone
%              wheels,...  % The wheels
%              mainpos,...  % The main point
%              guidep,...  % The guide point
%              agvline,...  % The AGV line 
%                           % (line betweem the guige and main point)
%              sens];  % The ultrasonic sensors
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 5,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(path) ~= 3
  error('Invalid path parameters!');
end

if length(pos) ~= 5
  error('Invalid AGV initial position!');
end

if length(agvobj) ~= 7
  error('Invalid AGV object!');
end

if length(pathobj) ~= 1
  error('Invalid path object!');
end

  % Init the axis min and max values

  % Parameters of the AGV, path and initial position (for the axis)

xk = pos(1);  % the main point of the AGV

lagv = agv(5);  % length of the AGV

r = path(2);  % radius
ly = path(3);  % length of the last streight

set(pathobj,'visible','off');  % Do not show the path (resizing)

  % xmin, xmax, ymin, ymax of the axis

%axis([xk-lagv./2 r+lagv./2 -r-lagv./2 ly+lagv./10]);  % old version (path tr., coll. avioid.)

pr = 1.1;  % axes area aspect ratio, x/y

if r==0
  lwt=((ly+lagv./2+lagv./10).*pr)./2;
  axis([-lwt,lwt,-lagv./2,ly+lagv./10]);
else
  lwt = (ly+r+lagv./2+lagv./10).*pr-(-xk+r+lagv);
  axis([xk-lagv./2,r+lagv./2+lwt,-r-lagv./2,ly+lagv./10]);
end  % if

  % Plot the path and the AGV on the screen in its initial position

  % Plot the AGV on screen in its initial position

plotagv(agv,[pos(1),pos(2),pos(3)],agvobj);

  % Draw the path

pathp=gpathp(path,.05);  % Generate the points of the path

set(pathobj,...
  'visible','on',...
  'xdata',pathp(:,1),'ydata',pathp(:,2));  % Plot the path

drawnow;  % Draw the figure

