function [rtime,rpos,re,fspos]=...
  rplotagv(agv,sim,path,obst,statpar,pos,agvobj,pathobj,unspo,SliderSHndl,autChHndl)
%RPLOTAGV: Running AGV on screen
%
%     [rtime,rpos,re,fspos]=
%          rplotagv(agv,sim,path,obst,statpar,pos,agvobj,pathobj,unspo)
%
%          Where agv is the description of the vehicle,
%          sim is the collection of the simulation parameters,
%          path is the description of the path,
%          obst is the description of the obstacles,
%          statpar is the description of the docking station,
%          pos is the description of the initial agv positon,
%          agvobj is the AGV object,
%          pathobj is the path object,
%          unspo is the unsecure point object,
%          SliderSHndl is the state display slider handle
%          autChHndl is the auto on off checkbox handle
%          rtime is the recorded running time,
%          rpos is the recorded positions,
%          re is the recorded control parameters,
%          fspos is the first position 
%                fits the requirements of the station.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          sim:
%
%            dt [s]: time slice of the simulation
%            m [kg]: full mass of the AGV
%            fmax [N]: absolute limit for the 
%                      accelerating force of the wheel
%            vmax [m/s]: absolute limit for the speed of the AGV
%            ekmax [m]: absolute limit for the distance between
%                       the main point and the path
%
%          path:
%
%            lx : length of the first streight
%            r  : radius
%            ly : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          obst:
%
%            x,y : position of the obstacle
%            r  : radius of the obstacle
%
%          statpar:
%
%            demax : max error as a distance of the path 
%                    and the main point of the AGV 
%                    in the docking station
%            aemax : max error of the angle of the AGV 
%                    in the docking station
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          agvobj:
%
%            agvobj=[body,...  % Body of the AGV
%              guidezone,...  % The guide zone
%              wheels,...  % The wheels
%              mainpos,...  % The main point
%              guidep,...  % The guide point
%              agvline,...  % The AGV line 
%                           % (line betweem the guige and main point)
%              sens];  % The ultrasonic sensors
%
%          rtime:
%
%            [rtime] = [rtime;[time]];  % running time
%
%          rpos:
%
%            [rpos]=[rpos;[pos(1),pos(2),pos(3)]];  % positions
%
%          re:
%
%            [re] = [re;[ev,ek,vd,va,pos(4),pos(5)]];  % control parameters
%
%          fspos:
%
%            fspos = [pos(1),pos(2),pos(3),ev,ek,time];  % first station
%            fspos = [0,0,0,0,0,-1];  % if there are no accaptable station
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 11,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(sim) ~= 5
  error('Invalid simulation parameters!');
end

if length(path) ~= 3
  error('Invalid path parameters!');
end

if length(statpar) ~= 2
  error('Invalid docking station parameters!');
end

if length(pos) ~= 5
  error('Invalid AGV initial position!');
end

if length(agvobj) ~= 7
  error('Invalid AGV object!');
end

  % Modify the user interface

axHndl=gca;  % The axes object
figNumber = gcf;  % The figure

hndlList = get(figNumber,'UserData');

startHndl = hndlList(1);
stopHndl = hndlList(2);
closeHndl = hndlList(3);
editHndl = hndlList(4);
plotHndl = hndlList(7);
stateHndl = hndlList(9);

  % Switch of the previous unsecure points

set(unspo,'visible','off');

  % Modify the buttons

set([startHndl closeHndl editHndl plotHndl],'Enable','off');
set(stopHndl,'Enable','on');
stateHndl = hndlList(9);
set(stateHndl,'String','Running',...
  'ForegroundColor','g');

  % Information regarding the run status will be held in
  % the axis user data according to the following table:

run= 1;
stop=-1;

set(axHndl, ...
  'Userdata',run, ...  % Running
  'Drawmode','fast', ...
  'Visible','off', ...  % Do not show axes
  'NextPlot','add');

  % Refresh the visibility of the path

set(pathobj,'visible','off');
set(pathobj,'visible','on');

  % Initialize parameters

w = agv(1);  % the distance of the guide and the main point of the agv
ds = agv(3);  % widt of the path sensing

dt = sim(1);  % dt [s]: time slice of the simulation
vmax = sim(4);  % absolute limit for the speed of the AGV
ekmax = sim(5);  % absolute limit for the main distance

wagv = agv(4);  % width of the AGV

  % Parameters of the observation normalisation

rlmax = wagv;  % max distance of the left ultrasonic sensor
rrmax = wagv;  % max distance of the rightt ultrasonic sensor
rmmax = wagv;  % max distance of the middle ultrasonic sensor

  % max error of the path main point distance in the station

demax = statpar(1);

  % max error of the angle of the AGV in the station

aemax = statpar(2);

va = vmax;  % initial parameters
vd = 0;
ev = 0;

nvd = vd./vmax;  % init normalisation for recording the values
nva = va./vmax;

  % Initialize the unsecure point record parameters

unsprate = 5;  % rate of the unsecure point collecting

unspcnt = 0;  % init the unsecure point counter

  % Initialize other variables

rpos = [];  % clear position recording
re = [];  % clear other parameter recording

runsp = [];  % clear unsecure point recording

orl = nan;  % clear unsecure point temp variables

ly = path(3);  % length of the last streight
lim = ly-w;    % position limit for simulation

axXLim = get(axHndl,'Xlim');  % axes X limits
axYLim = get(axHndl,'Ylim');  % axes Y limits

  % Init the collision flag

coll = 0;  % coll=1 if collides (coll=0 if not)

  % clear the first position fits the requirements of the station

fspos = [0,0,0,0,0,-1];

time = 0;  % running time
rtime = [];  % running time

  % Generate the FLC (based on the vague environment 
  % of the Fuzzy Rulebase) parameters

[UD,VED,Rva,Rvd]=gagvflcc;  % Path tracking and collision avoidance
[UDC,VEDC,RCSva,RCSvd,RCRva,RCRvd,RCLva,RCLvd]=gagvflcf;  % Collision avoidance
[UDM,VEDM,Rmva,Rmvd]=gagvflcm;  % Metacontrol
[UDA,VEDA,RAc,RAs,RAr,RAl]=gagvflca;  % Fuzzy automata

  % antecedents and a single conlusion va

UDa = [UD(1:5,:);UD(8,:)];
VEDa = [VED(1:5,:);VED(8,:)];

UDCa = [UDC(1:3,:);UDC(6,:)];
VEDCa = [VEDC(1:3,:);VEDC(6,:)];

UDMa = [UDM(1:4,:);UDM(5,:)];
VEDMa = [VEDM(1:4,:);VEDM(5,:)];

  % antecedents and a single conlusion vd

UDd = [UD(1:7,:);UD(9,:)];
VEDd = [VED(1:7,:);VED(9,:)];

UDCd = [UDC(1:5,:);UDC(7,:)];
VEDCd = [VEDC(1:5,:);VEDC(7,:)];

UDMd = [UDM(1:4,:);UDM(6,:)];
VEDMd = [VEDM(1:4,:);VEDM(6,:)];

  % initial state of the automat

sa = [1,0,0,0];  % sc, ss, sr, sl

autChMin = get(autChHndl,'Min');
autChMax = get(autChHndl,'Max');
autChVal = get(autChHndl,'Value');

  % The main cycle
  % (running till not stopped or collides or running out the path)

%while (get(axHndl,'Userdata')==run) & pos(2)<lim & abs(ev)<=ds./2 & coll~=1  % old system
while (get(axHndl,'Userdata')==run) & ...
    pos(1)>axXLim(1) & pos(1)<axXLim(2) & pos(2)>axYLim(1) & pos(2)<lim & coll~=1

  time = time+dt;  % increasing the running time
  [rtime] = [rtime;[time]];  % running time

    % The new position

  [npos,ev,ek,rl,rr,rm]=agvsysca(agv,sim,path,obst,pos,va,vd);

    % Draw the running AGV

  plotagv(agv,[npos(1),npos(2),npos(3)],agvobj);

    % Refresh the unsecure point matrix

  unspcnt = unspcnt + 1;  % increase the unsecure point counter

  if unspcnt == unsprate

    unspcnt = 0;  % reset the counter

    if ~isnan(orl)  % there were previous observation

      [unspl]=geunsecp(orl,rl,rm,agv,npos);  % generate the unsecure points
      [unspr]=geunsecp(orr,rr,rm,agv,npos);
      [unspm]=geunsecp(orm,rm,rm,agv,npos);
      
      % drl=(orl(1)-rl(1))./dt;  % Ultrasonic distance changes
      % drr=(orr(1)-rr(1))./dt;
      % drm=(orm(1)-rm(1))./dt;

    else  % there were no previous observation

      [unspl]=[];
      [unspr]=[];
      [unspm]=[];

      % drl=NaN;  % Ultrasonic distance changes
      % drr=NaN;
      % drm=NaN;

    end  % if

      % add the new unsecure points to front of the record
      % and clean up the unsecure points record

    [unsp] = [unspl;unspr;unspm];

    [runsp] = clrunsp(unsp,runsp,rl,rr,rm,agv,npos);

      % Plot unsecure points

    [msrunsp,nsrunsp] = size(runsp);

    set(unspo,'visible','off');  % switch off the unsecure points

    if msrunsp > 0  % if it is not empty, plot

      set(unspo,'visible','on',...
        'xdata',runsp(:,1)','ydata',runsp(:,2)');

    end  % if

  end  % if

  orl = rl;  % refresh previous values
  orr = rr;
  orm = rm;

    % Check the collision

  [coll]=touch(agv,obst,npos);  % coll=1 if collides (coll=0 if not)

    % Calculate the max turn angle without side touch (collision)

  [alm,arm] = sidetcha(runsp,agv,pos);  % NaN if not exists

    % If the max turn angle without side collision not exists
    % substitute by pi/2

  if isnan(alm), alm=pi./2; end;
  if isnan(arm), arm=pi./2; end;

    % normalization of the inputs of the FLC

  nev = normvar(ds./2,ev);  % normalization and limitation of ev
  nek = normvar(ekmax,ek);  % normalization and limitation of ek

  nrl = normvar(rlmax,rl(1));  % normalization and limitation of rl
  nrr = normvar(rrmax,rr(1));  % normalization and limitation of rr
  nrm = normvar(rmmax,rm(1));  % normalization and limitation of rm

  nalm = normvar(pi./2,alm);  % normalization and limitation of alm
  narm = normvar(pi./2,arm);  % normalization and limitation of arm

    % The observations

  xd = [nev,nek,nrl,nrr,nrm,nalm,narm];  % Path Tracking and collision avoidance
  xa = [nev,nek,nrl,nrr,nrm];

  xcd = [nrl,nrr,nrm,nalm,narm];  % Collision avoidance
  xca = [nrl,nrr,nrm];

  if isnan(nev)
    pv = 0;  % path not valid
  else
    pv = 1;  % path valid
  end

  xsa = [sa,[nev,pv,nrl,nrr,nrm,nalm,narm]];  % observation of the automata

    % Crisp input, crisp output with interpolation in vague environment

  % Path tracking and collision avoidance
  nvdc = vagconcl(UDd,VEDd,Rvd,2,xd);  % 7 antecedent universes
  nvac = vagconcl(UDa,VEDa,Rva,2,xa);  % 3 antecedent universes

  % Collision avoidance
  nvds = vagconcl(UDCd,VEDCd,RCSvd,2,xcd);  % 5 antecedent universes
  nvas = vagconcl(UDCa,VEDCa,RCSva,2,xca);  % 3 antecedent universes

  % Collision avoidance and right tendency
  nvdr = vagconcl(UDCd,VEDCd,RCRvd,2,xcd);  % 5 antecedent universes
  nvar = vagconcl(UDCa,VEDCa,RCRva,2,xca);  % 3 antecedent universes

  % Collision avoidance and left tendency
  nvdl = vagconcl(UDCd,VEDCd,RCLvd,2,xcd);  % 5 antecedent universes
  nval = vagconcl(UDCa,VEDCa,RCLva,2,xca);  % 3 antecedent universes

  % Modifying the rulbase by the actual conclusions (Metacontrol)
  % Rm =  [ S1,S2,S3,S4,CSx;]
  % Rmva = [ l, z, z, z,      l;
  %          z, l, z, z,      l;
  %          z, z, l, z,      l;
  %          z, z, z, l,      l];

  Rmva(1,5) = nvac;
  Rmva(2,5) = nvas;
  Rmva(3,5) = nvar;
  Rmva(4,5) = nval;

  Rmvd(1,5) = nvdc;
  Rmvd(2,5) = nvds;
  Rmvd(3,5) = nvdr;
  Rmvd(4,5) = nvdl;

  % Fuzzy automata - new states

  sa(1) = vagconcl(UDA,VEDA,RAc,2,xsa);  % sc, path tracking and collision avoidance
  sa(2) = vagconcl(UDA,VEDA,RAs,2,xsa);  % ss, collision avoidance
  sa(3) = vagconcl(UDA,VEDA,RAr,2,xsa);  % sr, collision avoidance with right tendeny
  sa(4) = vagconcl(UDA,VEDA,RAl,2,xsa);  % sl, collision avoidance with left tendency

  n = 4;  % number of the states
  if get(autChHndl,'Value')==autChMax  % Automatic control
    % Adjust the state display sliders
    for i=1 : 1 : n  % Adjusting the State Sliders
      set(SliderSHndl(i),'Value',sa(i));
    end;  % for
  else  % Manual State adjustment
    for i=1 : 1 : n  % Reading the manual State values from the slider
      sa(i) = get(SliderSHndl(i),'Value');
    end;  % for
  end  % if

    % recording for the graph

  [re] = [re;[nev,nek,nrl,nrr,nrm,nalm,narm,nvd,nva,sa]];

  % Metacontrol
  nvd = vagconcl(UDMd,VEDMd,Rmvd,2,sa);  % 4 antecedent universes
  nva = vagconcl(UDMa,VEDMa,Rmva,2,sa);  % 4 antecedent universes

    % the results of the normalized conclusions (output for the FLC)

  vd = nvd.*vmax;  % get vd from the normalized conclusion
  va = nva.*vmax;  % get va from the normalized conclusion

  pos = npos;  % next step

  [rpos]=[rpos;[pos(1),pos(2),pos(3)]];  % recording for the graph

  if abs(pos(3))<=aemax & abs(ek)<=demax & pos(2)>=0 & fspos(6)<0

      % first position fits the requirements of the station
    fspos = [pos(1),pos(2),pos(3),ev,ek,time];

  end % if

  if ( abs(pos(3))>aemax | abs(ek)>demax ) & fspos(6)>-1

      % clear the first position fits the requirements of the station
    fspos = [0,0,0,0,0,-1];

  end % if

end % while

  % Modify the user interface

  % Modify the buttons

set([startHndl closeHndl editHndl plotHndl],'Enable','on');
set(stopHndl,'Enable','off');

if get(axHndl,'Userdata')~=run
  set(stateHndl,'String','Stopping',...
    'ForegroundColor','w');
end  % if

if pos(2)>=lim
  set(stateHndl,'String','Ended',...
    'ForegroundColor','g');
end  % if

if isnan(ev)
  set(stateHndl,'String','Path lost',...
    'ForegroundColor','r');
end  % if

if coll==1
  set(stateHndl,'String','Collision',...
    'ForegroundColor','r');
end  % if

