%RUNAGV: RUN the AGV simulation
%           Fuzzy Path Tracking for Automated Guided Vehicle
%              by interpolation in vague environment
%
%          Where agv is the description of the vehicle,
%          sim is the parameters of the simulation,
%          path is the description of the path,
%          pos is the description of the agv's positon,
%          ev is the distance between the guide point and the path
%          ek is the distance between the main point and the path
%          va,vd is the average and the differential speed
%                of the AGV's left and right drive
%
%          agv:
%
%            w  : distance of the sensor from the main point of the AGV
%            d  : distance of the wheels
%            ds : widt of the path sensing
%                 (ds/2 is the absolute limit for the distance
%                  between the guide point and the path (ev))
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          sim:
%
%            dt    : time slice of the simulation
%            magv  : full mass of the AGV
%            fmax  : absolute limit for the accelerating force of the wheel
%            vmax  : absolute limit for the speed of the AGV
%            ekmax : absolute limit for the
%                    distance between the main point and the path (ek)
%
%          path:
%
%            lx    : length of the first streight
%            rpath : radius
%            ly    : length of the last streight
%
%                               |
%                               | ly
%                               |
%                           r  /
%                      lx     /
%                   _________/
%
%          statpar:
%
%            demax : max error as a distance of the path 
%                    and the main point of the AGV 
%                    in the docking station
%            aemax : max error of the angle of the AGV 
%                    in the docking station
%
%          pos, npos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%
%          dpos: change of the position
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%
%          fspar:
%                % evs, evm, evl: ev small, middle, large;
%                % eks, ekm, ekl: ek small, middle, large
%                % vds, vdm     : vd small, middle;
%                % fvas, fvam   : va small, middle
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

% echo off
clc;

  % Simulation parameters

  % magv [kg]: full mass of the AGV

magv = 570;  % unloaded, 1570 full loaded

  % rpath [m] the radius of the path ( if rpath==0, straight only )

rpath = 1;

  % obstacle parameters

  % size of obstacles

obssize = 1;

  % maximum number of obstacles

maxobsnum = 5;  % max number of obstacles

  % Generating the simulation parameters

[agv,sim,path,statpar,pos]=iniagvsp(magv,rpath);

  % User interface for simulation, Initialize the Figure and the Axes

figNumber = figure(...
	'Name','Automated Guided Vehicle',... 
	'NumberTitle','off',...
	'Color',[0 0 0],...
	'BackingStore','off',...
	'NextPlot','add',...
	'Pointer','arrow',...
	'Units','normal',...
	'Position',[0 0 1 1],...
	'Resize','on',...
	'Visible','off');

axes( ...
  'Units','normalized', ...
  'Position',[0.05 0.05 0.75 0.95], ...
  'NextPlot','add',...
  'Visible','off');

axis('equal'); % equal axis

  % Information regarding the run status will be held in
  % the axis user data according to the following table:

run= 1;
stop=-1;

  % Information for all buttons

labelColor=[0.8 0.8 0.8];
yInitPos=0.90;
xPos=0.85;
btnLen=0.10;
btnWid=0.07;
fonts=12;  % Font Size

  % Spacing between the button and the next command's label

spacing=0.02;

  % The CONSOLE frame

frmBorder=0.02;
yPos=0.05-frmBorder;
frmPos=[xPos-frmBorder yPos btnLen+2*frmBorder 0.9+2*frmBorder];
h=uicontrol( ...
  'Style','frame', ...
  'Units','normalized', ...
  'Position',frmPos, ...
  'BackgroundColor',[0.50 0.50 0.50]);
    
  % The START button

yPos=0.90;
labelStr='Start';
cmdStr='start';

callbackStr=...
  ['[obst]=genobst(obsobj);',...
  '[rtime,rpos,re,fspos]=',...
  'rplotagv(agv,sim,path,obst,statpar,pos,agvobj,pathobj,unspo,SliderSHndl,autChHndl);'];

  % Generic button information

btnPos=[xPos yPos-spacing btnLen btnWid];
startHndl=uicontrol( ...
  'Style','pushbutton', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'String',labelStr, ...
  'Interruptible','on', ...
  'FontSize',fonts,...
  'Callback',callbackStr);

  % The State signal

yPos=yPos-(btnWid+spacing)+btnWid./2;

btnPos=[xPos yPos-spacing btnLen btnWid./2];  % label
stateHndl=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'ForegroundColor','w',...
  'Enable','on', ...
  'FontSize',fonts,...
  'FontWeight','bold',...
  'String','Stopping');

  % The STOP button

yPos=yPos-(btnWid+spacing);
labelStr='Stop';

  % Setting usercontrol to stop (=-1) will stop the run.

callbackStr='set(gca,''Userdata'',stop)';
    
  % Generic  button information

btnPos=[xPos yPos-spacing btnLen btnWid];
stopHndl=uicontrol( ...
  'Style','pushbutton', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'String',labelStr, ...
  'FontSize',fonts,...
  'Callback',callbackStr);

  % The EDIT button

yPos=yPos-(btnWid+spacing);
labelStr='Edit';

  % Setting 

callbackStr='ediagvsp';
    
  % Generic  button information

btnPos=[xPos yPos-spacing btnLen btnWid];
editHndl=uicontrol( ...
  'Style','pushbutton', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'String',labelStr, ...
  'FontSize',fonts,...
  'Callback',callbackStr);

  % The path radius slider

yPos=yPos-(btnWid+spacing);

  % Setting the radius of the path

callbackStr = [...
  '[agv,sim,path,statpar,pos]=modpathr(sim,agvobj,pathobj);',...
  'set(radiusvalue,''String'',get(radiusHndl,''Value''))'];

  % Generic  slider information

btnPos=[xPos yPos-spacing btnLen btnWid./2];  % slider
radiusHndl=uicontrol( ...
  'Style','slider', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'Min',0,...
  'Max',5,...
  'Value',path(2),...  % rpath : radius
  'FontSize',fonts,...
  'Callback',callbackStr);

btnPos=[xPos yPos-spacing+btnWid./2 btnLen./4 btnWid./2];  % label
radiuslabel=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String','R:');

btnPos=[xPos+btnLen./4 yPos-spacing+btnWid./2,...
  3.*btnLen./4 btnWid./2];  % value
radiusvalue=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String',num2str(get(radiusHndl,'Value')));

  % The mass of the AGV slider

yPos=yPos-(btnWid+spacing);

  % Setting the mass of the AGV

callbackStr = [...
  'magv = round(get(massHndl,''Value'')./10).*10;',...
  'set(massHndl,''Value'',magv);',...
  'sim(2) = magv;',...
  'set(massvalue,''String'',get(massHndl,''Value''));'];

  % Generic  slider information

btnPos=[xPos yPos-spacing btnLen btnWid./2];  % slider
massHndl=uicontrol( ...
  'Style','slider', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'Min',570,...  % unloaded 570 [kg]
  'Max',1570,...  % full loaded 1570 [kg]
  'Value',sim(2),...  % magv : full mass of the AGV
  'FontSize',fonts,...
  'Callback',callbackStr);

btnPos=[xPos yPos-spacing+btnWid./2 btnLen./4 btnWid./2];  % label
masslabel=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String','M:');

btnPos=[xPos+btnLen./4 yPos-spacing+btnWid./2,...
  3.*btnLen./4 btnWid./2];  % value
massvalue=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String',num2str(get(massHndl,'Value')));

  % The maximal speed of the AGV slider

yPos=yPos-(btnWid+spacing);

  % Setting the maximal speed of the AGV

callbackStr = [...
  'vmax = get(speedHndl,''Value'');',...
  'rvmax = round(vmax.*10)./10;',...
  'if rvmax<vmax & rvmax<1.5, vmax=rvmax+.1;',...
  'elseif rvmax>vmax & rvmax>0, vmax=rvmax-.1;',...
  'else vmax = rvmax; end;',...
  'set(speedHndl,''Value'',vmax);',...
  'sim(4) = vmax;',...
  'set(speedvalue,''String'',get(speedHndl,''Value''));'];

  % Generic  slider information

btnPos=[xPos yPos-spacing btnLen btnWid./2];  % slider
speedHndl=uicontrol( ...
  'Style','slider', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'Min',0,...  % stopping
  'Max',1.5,...  % maximal speed 1.5 [m/s]
  'Value',sim(4),...  % vmax : maximal speed of the AGV
  'FontSize',fonts,...
  'Callback',callbackStr);

btnPos=[xPos yPos-spacing+btnWid./2 btnLen./2 btnWid./2];  % label
speedlabel=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String','Vm:');

btnPos=[xPos+btnLen./2 yPos-spacing+btnWid./2,...
  btnLen./2 btnWid./2];  % value
speedvalue=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String',num2str(get(speedHndl,'Value')));

  % The Obstacles slider

yPos=yPos-(btnWid+spacing);

  % Setting the number of the obstacles

callbackStr = [...
  'obstnum = get(obstHndl,''Value'');',...
  'robstnum = round(obstnum);',...
  'if robstnum<obstnum & robstnum<maxobsnum, obstnum=robstnum+1;',...
  'elseif robstnum>obstnum & robstnum>0, obstnum=robstnum-1;',...
  'else obstnum = robstnum; end;',...
  'set(obstHndl,''Value'',obstnum);',...
  'set(obstvalue,''String'',get(obstHndl,''Value''));',...
  'refrobst(obsobj);'];

  % Generic  slider information

btnPos=[xPos yPos-spacing btnLen btnWid./2];  % slider
obstHndl=uicontrol( ...
  'Style','slider', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'Min',0,...  % no obstacles
  'Max',maxobsnum,...  % max number of obstacles
  'Value',0,...  % no obstacles
  'FontSize',fonts,...
  'Callback',callbackStr);

btnPos=[xPos yPos-spacing+btnWid./2 4.*btnLen./5 btnWid./2];  % label
obstlabel=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String','Obstacle:');

btnPos=[xPos+4.*btnLen./5 yPos-spacing+btnWid./2,...
  btnLen./5 btnWid./2];  % value
obstvalue=uicontrol( ...
  'Style','text', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','on', ...
  'FontSize',fonts,...
  'String',num2str(get(obstHndl,'Value')));

  % The Plot button

yPos=yPos-(btnWid+spacing);
labelStr='Plot';

  % Setting 

callbackStr='plagvres(agv,sim,path,obst,rtime,rpos,re,fspos);';
    
  % Generic  button information

btnPos=[xPos yPos-spacing btnLen btnWid];
plotHndl=uicontrol( ...
  'Style','pushbutton', ...
  'Units','normalized', ...
  'Position',btnPos, ...
  'Enable','off', ...
  'String',labelStr, ...
  'FontSize',fonts,...
  'Callback',callbackStr);

  % The CLOSE button

labelStr='Close';
callbackStr='close(gcf)';
closeHndl=uicontrol( ...
  'Style','push', ...
  'Units','normalized', ...
  'position',[xPos 0.05 btnLen btnWid], ...
  'string',labelStr, ...
  'FontSize',fonts,...
  'call',callbackStr);

  % The actual state sliders

sliderStrS = ['uicontrol(''Parent'',figNumber,', ...
    '''Units'',''normalized'',', ...
    '''ListboxTop'',0,', ...
    '''Style'',''slider'',', ...
    '''Min'',0,', ...
    '''Max'',1,', ...
    '''Enable'',''inactive'','];

sliderw = .01;  % Width of the state sliders
sliderl = 0.9./3;  % Length of the state sliders
x = xPos-frmBorder-4.5*sliderw;
y = 0.05;
n = 4;  % number of the states

for i=1 : 1 : n  % Generating the State values sliders
  eval(['SliderS',int2str(i),'=',sliderStrS,...
      '''Value'',0,',...  % Initial value
      '''Position'',[x+(i-1).*sliderw,y,sliderw,sliderl]);']);
end;  % for

set(SliderS1,'Value',1);  % Initial value (Path tracking)

SliderSHndlStr = [];  % init
for i=1 : 1 : n  % Generating the User State values Slider Handles
  SliderSHndlStr = [SliderSHndlStr,'SliderS',int2str(i),','];  % add to the end
end;  % for

eval(['SliderSHndl=[',SliderSHndlStr,'];']);  % evaluate

sltextLab = [ ...  % User State values Slider Label Values
    'P'; ...
    'C'; ...
    'R'; ...
    'L'];

sltextCol = [ ...  % User State values Slider Label Colors
    [1 1 0]; ...  % Yellow
    [0 0 1]; ...  % Blue
    [0 1 0]; ...  % Green
    [1 0 0]];  % Red

sltextStr = ['uicontrol(''Parent'',figNumber,', ...  % User State values Slider Labels
    '''Style'',''text'',', ...
    '''Units'',''normalized'',', ...
    '''FontUnits'',''normalized'',', ...
    '''FontSize'',sliderw.*.66,', ...
    '''FontWeight'',''normal'',', ...
    '''ForegroundColor'',[0 0 0],', ...  % black
    '''HorizontalAlignment'',''center'',', ...
    '''Visible'',''on'','];

for i=1 : 1 : n  % Generating the User State values Slider Labels
  eval([sltextStr, ...
      '''Position'',[x+(i-1).*sliderw,y+sliderl,sliderw,sliderw.*2],', ...
      '''BackgroundColor'',sltextCol(i,:),', ...
      '''String'',sltextLab(i))']);
end;  % for   

  % The automatic state adjustment on/off checkbox

callbackStr = [...
  'autChVal = get(autChHndl,''Value'');',...
  'if autChVal==1;',...
  'for i=1 : 1 : n;',...  % Inactivating the User State values Sliders
  'set(SliderSHndl(i),''Enable'',''inactive'');',...
  'end;',...  % for
  'else;',...
  'for i=1 : 1 : n;',...  % Activating the User State values Sliders
  'set(SliderSHndl(i),''Enable'',''on'');',...
  'end;',...  % for
  'end;'];  % if

autChHndl = uicontrol('Parent',figNumber, ...
     'Units','normalized', ...
     'ListboxTop',0, ...
     'Style','checkbox', ...
     'Min',0, ...
     'Max',1, ...
     'Value',1, ...
     'String','Auto', ...
     'Position',[x-sliderw.*.5,y-sliderw.*2.5,sliderw.*4.5,sliderw.*2],...
     'Enable','on', ...
     'Callback',callbackStr);

  % Generate the Figure object

cla;  % clear axis

  % generate the obstacle object

obsobj=initobso(obssize,maxobsnum);

  % Generate the AGV object

agvobj = initagvo;  % Init the AGV object

  % Generate the Path object

pathobj = line( ...
  'color','y', ...
  'linestyle','-', ...
  'visible','off',...
  'xdata',0,'ydata',0);  % The path

%  'erase','xor', ...  % old version (Matlab 4)

  % Init the unsecure points plot object

unspo = line( ...
  'color','b', ...
  'linestyle','none', ...
  'marker','o', ...
  'visible','off',...
  'erase','xor', ...
  'xdata',0,'ydata',0);

  % Plot the path and the AGV on the screen in its initial position

plotpagv(agv,path,pos,agvobj,pathobj);

  % Uncover the figure

hndlList=[...  % Handles
  startHndl,...  % hndlList(1)
  stopHndl,...  % hndlList(2)
  closeHndl,...  % hndlList(3)
  editHndl,...  % hndlList(4)
  radiusHndl,...  % hndlList(5)
  massHndl,...  % hndlList(6)
  plotHndl,...  % hndlList(7)
  obstHndl,...  % hndlList(8)
  stateHndl,...  % hndlList(9)
  speedHndl,...  % hndlList(10)
  obsobj];  % hndlList(11) - hndlList(9+maxobsnum)

set(figNumber,'Visible','on', ...
  'UserData',hndlList);

  % set the WindowButton handler

set(figNumber,'WindowButtonDownFcn','moveobj');
set(figNumber,'WindowButtonUpFcn','moveobj');

  % Draw the figure

figure(figNumber);  % Draw the figure

