function [almax,armax] = sidetcha(runsp,agv,pos)
%SIDETCHA: Calculate the max turn angle without side touch (collision)
%
%          [almax,armax] = sidetcha(runsp,agv,pos)
%
%          Where almax,armax is the left and right
%            maximal turn angle withouth side collision,
%          runsp is the unsecure points matrix,
%          agv is the description of the vehicle,
%          pos is the description of the agv positon.
%
%          runsp, nrunsp matrix of:
%
%            x,y : position of the approximated unsecure points
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 5
  error('Invalid AGV position!');
end

  % Parameters of the AGV

wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

  % Position of the AGV

x0 = pos(1);  % position of the main point of the AGV
y0 = pos(2);
fi = pos(3);  % angle of the AGV

  % local variables

hw = wagv./2;  % half width of the AGV
hl = lagv./2;  % half length of the AGV

rs = hw.^2+hl.^2;  % distance^2 of the main point and the corner

  % The mean cycle

[m,n] = size(runsp);

ralmax = [];  % clear the new record
rarmax = [];

if m > 0  % the record is not empty

  for i=1 : m  % check all the points

    x = runsp(i,1);  % the actual unsecure point
    y = runsp(i,2);

    dx = x-x0;
    dy = y-y0;

    rus = dx.^2+dy.^2;  % distance^2 of the main point and the unsp

      % The unsp can be touched if

    if rus <= rs

      bet0 = atan2(dx,dy);  % atan(dx./dy) the angle of the point

      du = sqrt(rus - hw.^2);

      gam = atan2(hw,du);  % atan(hw./du) the angle inside the AGV

      bet = bet0-fi;  % angle relative to the AGV

        % Correcting bet to be between [-pi,pi]

      if bet>=pi, bet=bet-2.*pi; end;
      if bet<=-pi, bet=bet+2.*pi; end;

        % selecting one of the four possible positions

      if 0<=bet & bet<pi

          % the angle of the AGV touching x,y (the ith unsp)

        alf = bet - gam;

        if alf >= 0  % valid value (unsp not under AGV)

          rarmax = [rarmax,alf];  % add to the right record

        end  % if

      end  % if

      if -pi<=bet & bet<-pi./2

          % the angle of the AGV touching x,y (the ith unsp)

        alf = pi + bet - gam;

        if alf >= 0  % valid value (unsp not under AGV)

          rarmax = [rarmax,alf];  % add to the right record

        end  % if

      end  % if

      if -pi./2<=bet & bet<0

          % the angle of the AGV touching x,y (the ith unsp)

        alf = - bet - gam;

        if alf >= 0  % valid value (unsp not under AGV)

          ralmax = [ralmax,alf];  % add to the left record

        end  % if

      end  % if

      if pi./2<=bet & bet<pi

          % the angle of the AGV touching x,y (the ith unsp)

        alf = pi - bet - gam;

        if alf >= 0  % valid value (unsp not under AGV)

          ralmax = [ralmax,alf];  % add to the left record

        end  % if

      end  % if

    end  % if

  end  % for i

end  % if

[m,n] = size(ralmax);

if n>0  % there the max turn angle is exists

  almax = min(ralmax);  % the max turn angle without side collision

else  % not exists

  almax = nan;  % not a number

end  % if

[m,n] = size(rarmax);

if n>0  % there the max turn angle is exists

  armax = min(rarmax);  % the max turn angle without side collision

else  % not exists

  armax = nan;  % not a number

end  % if

