function [coll]=touch(agv,obst,pos)
%TOUCH: Calculate the collision to the obstacles
%
%          [coll]=touch(agv,obst,pos)
%
%          Where coll=1 if collides (coll=0 if not), 
%          agv is the description of the vehicle,
%          obst is the description of the obstacles,
%          pos is the description of the agv's positon.
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          obst:
%
%            x,y : position of the obstacle
%            r  : radius of the obstacle
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 5
  error('Invalid Position!');
end

  % Parameters of the AGV

wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

hw = wagv./2;  % half width of the AGV
hl = lagv./2;  % half length of the AGV

x0 = pos(1);  % position of the main point of the AGV
y0 = pos(2);
fi = pos(3);  % angle of the AGV

  % Positions of the corner points of the AGV

sinfi = sin(fi);
cosfi = cos(fi);

  % Front middle

xmf = x0 + hl.*sinfi;
ymf = y0 + hl.*cosfi;

  % Front left corner

xlf = xmf - hw.*cosfi;
ylf = ymf + hw.*sinfi;

  % Front right corner

xrf = xmf + hw.*cosfi;
yrf = ymf - hw.*sinfi;

  % Rear middle

xmr = x0 - hl.*sinfi;
ymr = y0 - hl.*cosfi;

  % Rear left corner

xlr = xmr - hw.*cosfi;
ylr = ymr + hw.*sinfi;

  % Rear right corner

xrr = xmr + hw.*cosfi;
yrr = ymr - hw.*sinfi;

  % Init the collision flag (no collision)

coll = 0;  % Init the collision flag (no collision)

  % Obstacles

[m,n] = size(obst);

if m > 0 & n ~= 3
  error('Invalid obstacle descriptor!');
end

if m > 0  % there are obstacles

  for i=1 : m  % check all obstacles

    x = obst(i,1);  % the ith obstacle
    y = obst(i,2);
    r = obst(i,3);

    rs = r.^2;


      % Left front

    dx = x-xlf;
    dy = y-ylf;

    a = dx.^2+dy.^2;  % distance^2 from the corner
    b = (x-xlr).^2+(y-ylr).^2;

    da = (a-b+lagv.^2)./(2.*lagv);  % projection to the side 
    m = a-da.^2;  % distance^2 from the side

    alf = atan2(dx,dy);  % atan(dx./dy) the angle
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

      % side collision

    if -pi<=ang & ang<=-pi./2 & 0<=da & da<=lagv & m<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if

      % corner collision

    if -pi./2<=ang & ang<=0 & a<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if


      % Right front

    dx = x-xrf;
    dy = y-yrf;

    a = dx.^2+dy.^2;  % distance^2 from the corner
    b = (x-xlf).^2+(y-ylf).^2;

    da = (a-b+wagv.^2)./(2.*wagv);  % projection to the side 
    m = a-da.^2;  % distance^2 from the side

    alf = atan2(dx,dy);  % atan(dx./dy) the angle
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

      % side collision

    if -pi./2<=ang & ang<=0 & 0<=da & da<=wagv & m<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if

      % corner collision

    if 0<=ang & ang<=pi./2 & a<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if


      % Right rear

    dx = x-xrr;
    dy = y-yrr;

    a = dx.^2+dy.^2;  % distance^2 from the corner
    b = (x-xrf).^2+(y-yrf).^2;

    da = (a-b+lagv.^2)./(2.*lagv);  % projection to the side 
    m = a-da.^2;  % distance^2 from the side

    alf = atan2(dx,dy);  % atan(dx./dy) the angle
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

      % side collision

    if 0<=ang & ang<=pi./2 & 0<=da & da<=lagv & m<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if

      % corner collision

    if pi./2<=ang & ang<=pi & a<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if


      % Left Rear

    dx = x-xlr;
    dy = y-ylr;

    a = dx.^2+dy.^2;  % distance^2 from the corner
    b = (x-xrr).^2+(y-yrr).^2;

    da = (a-b+wagv.^2)./(2.*wagv);  % projection to the side 
    m = a-da.^2;  % distance^2 from the side

    alf = atan2(dx,dy);  % atan(dx./dy) the angle
    ang = alf-fi;

      % Correcting ang to be between [-pi,pi]

    if ang>=pi, ang=ang-2.*pi; end;
    if ang<=-pi, ang=ang+2.*pi; end;

      % side collision

    if pi./2<=ang & ang<=pi & 0<=da & da<=wagv & m<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if

      % corner collision

    if -pi<=ang & ang<=-pi./2 & a<=rs

      coll = 1;  % Set the collision flag (collision)

    end  % if

  end  % for i

end  % if

