function [rl,rr,rm]=usense(agv,obst,pos)
%USENSE: Calculate the distances measured by the ultrasonic sensors
%
%          [rl,rr,rm]=usense(agv,obst,pos)
%
%          Where rl,rr,rm are measured distances of the left, 
%            right and middle ultrasonic sensors
%          agv is the description of the vehicle,
%          obst is the description of the obstacles,
%          pos is the description of the agv's positon.
%
%          rl,rr,rm:
%
%            r : measured distance
%            x,y : position of the measurement
%
%          agv:
%
%            w : distance of the sensor from the main point of the AGV
%            d : distance of the wheels
%            ds : widt of the path sensing
%            wagv : width of the AGV
%            lagv : length of the AGV
%
%          obst:
%
%            x,y : position of the obstacle
%            r  : radius of the obstacle
%
%          pos:
%
%            xk,yk : the main point of the AGV
%            fi    : angle of the AGV
%            vao   : previous speed
%            dfio  : previous angular velocity
%

% Fuzzy Path Tracking for Automated Guided Vehicle
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 3,error('Invalid number of arguments!');end

if length(agv) ~= 5
  error('Invalid AGV descriptor!');
end

if length(pos) ~= 5
  error('Invalid Position!');
end

  % Parameters of the AGV

wagv = agv(4);  % width of the AGV
lagv = agv(5);  % length of the AGV

hw = wagv./2;  % half width of the AGV
hl = lagv./2;  % half length of the AGV

x0 = pos(1);  % position of the main point of the AGV
y0 = pos(2);
fi = pos(3);  % angle of the AGV

  % Parameters of the ultrasonic sensors

usmax = lagv;  % Maximal sensing distance

  % Positions of the ultrasonic sensors

sinfi = sin(fi);
cosfi = cos(fi);

  % Middle sensor

xm = x0 + hl.*sinfi;
ym = y0 + hl.*cosfi;

  % Left sensor

xl = xm - hw.*cosfi;
yl = ym + hw.*sinfi;

  % Right sensor

xr = xm + hw.*cosfi;
yr = ym - hw.*sinfi;

  % Obstacles

[m,n] = size(obst);

if m > 0 & n ~= 3
  error('Invalid obstacle descriptor!');
end

if m > 0  % there are obstacles

  rli = [];  % clear
  rri = [];  % clear
  rmi = [];  % clear

  for i=1 : m  % check all obstacles

    x = obst(i,1);  % the ith obstacle
    y = obst(i,2);
    r = obst(i,3);

      % Calculating obstacle distances from the left sensor

    dx = x-xl;
    dy = y-yl;

    alf = atan2(dx,dy);  % atan((x-xl)./(y-yl)) the angle of the obstacle
    ang = alf-fi;
    if ang>pi  % Correcting ang to be between [-pi,pi]
      ang=ang-2.*pi;
    end  % if
    if ang<=-pi
      ang=ang+2.*pi;
    end  % if

    if -pi<=ang & ang<=pi./2  % the obstacle is visible

      rli(i) = sqrt(dx.^2+dy.^2)-r;  % the distance

      if rli(i) < 0  % lower limit

        rli(i) = 0;

      end  % if

      if rli(i) > usmax  % out of range

        rli(i) = inf;

      end  % if

    else  % not visible

      rli(i) = inf;

    end  % if

      % Calculating obstacle distances from the right sensor

    dx = x-xr;
    dy = y-yr;

    alf = atan2(dx,dy);  % atan((x-xr)./(y-yr)) the angle of the obstacle
    ang = alf-fi;
    if ang>pi  % Correcting ang to be between [-pi,pi]
      ang=ang-2.*pi;
    end  % if
    if ang<=-pi
      ang=ang+2.*pi;
    end  % if

    if -pi./2<=ang & ang<=pi  % the obstacle is visible

      rri(i) = sqrt(dx.^2+dy.^2)-r;  % the distance

      if rri(i) < 0  % lower limit

        rri(i) = 0;

      end  % if

      if rri(i) > usmax  % out of range

        rri(i) = inf;

      end  % if

    else  % not visible

      rri(i) = inf;
 
    end  % if

      % Calculating obstacle distances from the middle sensor

    dx = x-xm;
    dy = y-ym;

    alf = atan2(dx,dy);  % atan((x-xm)./(y-ym)) the angle of the obstacle
    ang = alf-fi;
    if ang>pi  % Correcting ang to be between [-pi,pi]
      ang=ang-2.*pi;
    end  % if
    if ang<=-pi
      ang=ang+2.*pi;
    end  % if

    if -pi./2<=ang & ang<=pi./2  % the obstacle is visible

      rmi(i) = sqrt(dx.^2+dy.^2)-r;  % the distance

      if rmi(i) < 0  % lower limit

        rmi(i) = 0;

      end  % if

      if rmi(i) > usmax  % out of range

        rmi(i) = inf;

      end  % if

    else  % not visible

      rmi(i) = inf;

    end  % if

  end  % for i

  rl = [min(rli),xl,yl];  % the measured value is the minimal distance
  rr = [min(rri),xr,yr];
  rm = [min(rmi),xm,ym];

else  % there are no obstacles

  rl = [inf,xl,yl];  % there are no obstacles
  rr = [inf,xr,yr];
  rm = [inf,xm,ym];

end  % if

