function [y]=vagconcl(u,ve,r,p,x)
%vagconcl: Calculate the conclusion from the observation and the rulebase
%
%                            [Y]=vagconcl(U,VE,R,P,X)
%
%          Where Y is the conclusion to calculate,
%          U is the universe, VE is the vague environment,
%          (the rows are the dimensions)
%          R is the rulebase, 
%          P is the power of the distance in the weight factor
%            ( wi = 1 ./ dist(i).^p; )
%          X is the observation.
%
%          Rulebase:
%
%            R1 | a1 a2 a3 ... am -> b
%            R2 | a1 a2 a3 ... am -> b
%            R2 | a1 a2 a3 ... am -> b
%             .
%             .
%             .
%            Rn | a1 a2 a3 ... am -> b
%

% Fuzzy Inference by Interpolation in Vague Environment toolbox for MATLAB
% By Szilveszter Kovacs
% e-mail: szkszilv@gold.uni-miskolc.hu
%   Copyright (c) by Szilveszter Kovacs

if nargin ~= 5,error('Invalid number of arguments!');end

if length(u) ~= length(ve)
  error('Input arguments must be the same length!');
end

dist = ruledist(u,ve,r,x);

[n,m] = size(r);

% Go through all rules; r(i,m) is the i. conclusion;
%    dist(i) is the i. distance of the i. antecedent to the observation

if min(dist) == 0

  vagc = 0;  % resetting temp variables
  nc = 0;

  for i=1 : n

    if dist(i) == 0

      vrconc = vagdist(u(m,:),ve(m,:),u(m,1),r(i,m));
      % vague distance from the first (smallest) element
      %   of the conclusion universe

      nc = nc + 1;
      vagc = vagc + vrconc;

    end % if

  end % for i

  vagc = vagc ./ nc;

else

  vagc = 0;  % resetting temp variables
  ws = 0;

  for i=1 : n
    vrconc = vagdist(u(m,:),ve(m,:),u(m,1),r(i,m));
    % vague distance from the first (smallest) element
    %   of the conclusion universe
    wi = 1 ./ dist(i).^p;
    ws = ws + wi;
    vagc = vagc + wi .* vrconc;
  end % for i

  if ws==0
    vagc = NaN;  % There are no valid conclusion (infinite distances from all the rules)
  else
    vagc = vagc ./ ws;  % The valid vague conclusion
  end % if

end % if

if isnan(vagc)
  y = NaN;  % There are no valid conclusion (infinite distances from all the rules)
else
  y = valvag(u(m,:),ve(m,:),vagc);  % The valid conclusion
end % if

